      program main
c
c  convert model given in standard ASCII GONG format into 
c  input model for adiabatic oscillation package.
c
c  Note: it is assumed that model corresponds to a complete 
c  model. Hence it is extended to zero radius, if that point is
c  not included.
c
c  If second derivatives are not set in glob(11) and glob(12),
c  they are estimated from the central values of the other
c  model variables
c
c  Original version: 25/7/97
c
c  Modified 29/11/11, setting data(6) from just the innermost few points,
c  if not provided in fgong file.
c
c  Modified 6/4/13, introducing input parameter iform to select between
c  formatted (iform = 1; default) and free-format (iform = 0) input
c
c  Modified 3/4/14, redefining iform = 10*iform1+iform0.
c  iform0 is used to set format as above
c  iform1 = 1 is used to flag for resetting A4 at Garstec spikes using
c  numerical density derivative
c
c  Modified 6/4/17, redefining iform = 100*iform2+10*iform1+iform0
c  iform2 = 1 is used to flag for eliminating points with identical r
c
c  Modified 26/11/17, refining the treatment of excessively large 
c  negative buoyancy frequency, including skipping resetting 
c  (with warning) when the region extends over many meshpoints
c  controlled by kbv_lim, hardcoded below.
c
      implicit double precision(a-h, o-z)
      parameter (nnmax=10000, iconmx=30, ivarmx=50, iaa=10)
      character*80 fin, fout,head
      dimension glob(iconmx), var(ivarmx,nnmax), var1(ivarmx,nnmax),
     *  data(8), aa(iaa,nnmax), ireset(16),q(nnmax),x(nnmax)
      data ireset /3,4,5,6,8,9,10,11,12,13,14,16,17,18,19,20/
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      cgrav=6.67232d-8
      pi4=16.d0*atan(1.d0)
      iform=1
c
c  Parameters for resetting buoyancy frequency
c
      bv_lim=-1.d0
      xbv_lim=0.9d0
      kbv_lim=10
c
      write(istdou,*) 'Enter input GONG file'
      read(istdin,'(a)') fin
      write(istdou,*) 'Enter output amdl file'
      read(istdin,'(a)') fout
      write(istdou,*) 
     *  'Enter gravitational constant (in units of 1.d-8 cgs)'
      read(istdin,*) cgrav
      cgrav=cgrav*1.d-8
      write(istdou,*) 
     *  'Enter atmospheric truncation point'
      read(istdin,*) xtrnc
      read(istdin,*,end=10) iform
c
   10 continue
c
      iform0=mod(iform,10)
      iform1=mod(iform/10,10)
      iform2=iform/100
c
      open(2,file=fin,status='old')
      open(3,file=fout,status='unknown',form='unformatted')
c
      if(istdpr.gt.0) write(istdpr,110)
      do 15 i=1,4
      read(2,'(a)') head
      if(istdpr.gt.0) write(istdpr,'(a)') head
   15 continue
c
      read(2,120) nn, iconst, ivar, ivers
c
c force free format if new, extended format is used
c
      if(ivers.gt.1000) iform0=0
c..      write(istdpr,*) '#D# nn, iconst, ivar, ivers, iform0',
c..     *  nn, iconst,ivar,ivers,iform0
      if(iform0.eq.1) then 
        read(2,130) (glob(i),i=1,iconst)
      else
        read(2,*) (glob(i),i=1,iconst)
      end if
c
      do n=1,nn
        if(iform0.eq.1) then 
          read(2,130) (var(i,n),i=1,ivar)
        else
          read(2,*) (var(i,n),i=1,ivar)
        end if
      end do
c
      if(var(1,1).gt.var(1,nn)) then 
	nn1=nn+1
	do 30 i=1,ivar
	do 25 n=1,nn
   25   var1(i,n)=var(i,nn1-n)
	do 30 n=1,nn
   30   var(i,n)=var1(i,n)
      end if
c
c  test for cgrav in fgong file
c
      if(iconst.ge.15) then
        if(glob(15).gt.0.and.glob(15).le.1.d-7) cgrav=glob(15)
      end if
      if(istdpr.gt.0) write(istdpr,'(/'' Using G ='',1pe13.5)') cgrav
c
      if(var(1,1).gt.1.e6) then 
        write(istdou,*) ' ***** Warning: extending to centre'
        if(istdpr.gt.0.and.istdpr.ne.istdou)
     *    write(istdpr,*)  ' ***** Warning: extending to centre'
	do 32 i=1,ivar
	do 32 n=1,nn
   32   var1(i,n+1)=var(i,n)
c
	do 34 i=1,ivar
   34   var1(i,1)=0
c
	do 36 ir=1,16
	i=ireset(ir)
   36   var1(i,1)=var1(i,2)
c
        nn=nn+1 
	do 38 i=1,ivar
	do 38 n=1,nn
   38   var(i,n)=var1(i,n)
      end if
c
      if(xtrnc.lt.0) xtrnc=1.d10
c
      nnt=nn
      do n=1,nn
        q(n)=exp(var(2,n))
        x(n)=var(1,n)/glob(2)
        if(x(n).le.xtrnc) nnt=n
      end do
c
      if(nnt.lt.nn) then
        write(istdpr,'(/'' Truncate model at n = '',i5,'' r/R ='',
     *    f12.7)') nnt, x(nnt)
        nn=nnt
      end if
c
      x(1)=0
      q(1)=0
c
c  test for eliminating identical points
c
      if(iform2.gt.0) then
c
        n=2
        do while(n.lt.nn) 
          if(x(n+1).le.x(n)) then
            if(istdpr.gt.0) write(istdpr,
     *        '(/'' Remove identical points. n, x, var(15,.):'')')
            k=1
            do while(x(n+k).le.x(n))
              if(istdpr.gt.0) write(istdpr,'(i5,1p2e20.12)')
     *          n+k,x(n+k),var(15,n+k)
              k=k+1
            end do
            if(istdpr.eq.0) write(istdou,
     *        '(/'' Remove identical points from n ='',i6,'' to'',i6)')
     *        n+1, n+k-1
            nn=nn-k+1
            do n1=n+1,nn
              n2=n1+k-1
              do j=1,ivar
                var(j,n1)=var(j,n2)
              end do
              x(n1)=x(n2)
              q(n1)=q(n2)
            end do
          end if
          n=n+1
        end do
      end if
c
      do 45 n=2,nn
      aa(1,n)=x(n)
      aa(2,n)=q(n)/x(n)**3
      aa(3,n)=cgrav*glob(1)*q(n)*var(5,n)/ 
     *        (var(10,n)*var(4,n)*var(1,n))
      aa(4,n)=var(10,n)
      aa(5,n)=var(15,n)
   45 aa(6,n)=pi4*var(5,n)*var(1,n)**3/(glob(1)*q(n))
c
      aa(1,1)=0
      aa(2,1)=pi4/3.d0*var(5,1)*glob(2)**3/glob(1)
      aa(3,1)=0
      aa(4,1)=var(10,1)
      aa(5,1)=0
      aa(6,1)=3.d0
c
c Possibly prepare for proper resetting of A4 in case of garstec spikes
c
      if(iform1.eq.1) then
        do n=1,nn
          aa(8,n)=dlog(var(5,n))
        end do
        call derive(x,aa(8,1),aa(7,1),nn,iaa,iaa,1,1)
        do n=1,nn
          aa(8,n)=-aa(3,n)-x(n)*aa(7,n)
        end do
      end if
c
c  test for Garstec spikes
c
      do n=3,nn
        if(aa(5,n).le.-1.d2*(aa(3,n-1)+aa(3,n+1)).or.
     *     aa(5,n).le.-1.d2*abs(aa(5,n-1)+aa(5,n+1))) then
c
c  reset, depending on iform1
c  in convection zone, use simple interpolation
c
          if(iform1.eq.0.or.aa(5,n-1).lt.0.or.aa(5,n+1).lt.0) then
            aa4_new=aa(5,n-1)+(aa(5,n+1)-aa(5,n-1))*
     *        (x(n)-x(n-1))/(x(n+1)-x(n-1))
            write(istdpr,'(/'' Garstec spike in A_4.''/
     *        '' n, x, A_2, A_4,orig, A_4,reset:''/
     *        (i5,1p4e13.5))') 
     *        n-1,x(n-1),aa(3,n-1),aa(5,n-1),aa(5,n-1),
     *        n,  x(n  ),aa(3,n  ),aa(5,n  ),aa4_new,
     *        n+1,x(n+1),aa(3,n+1),aa(5,n+1),aa(5,n+1)
            aa(5,n)=aa4_new
          else
c
c  reset from density derivative
c
            n1=max(2,n-2)
            n2=min(nn,n+2)
            write(istdpr,'(/'' Garstec spike in A_4.''/
     *        '' n, x, A_2, A_4,orig, A_4,reset:''/
     *        (i5,1p4e13.5))') (k, x(k),aa(3,k),aa(5,k),aa(8,k),k=n1,n2)
            do k=n1,n2
              aa(5,k)=aa(8,k)
            end do
          end if
        end if
      end do
c
c  test for remaining excessive negative squared buoyancy frequency
c
      ires=0
      ihead=1
      do n=2,nn
        if(aa(2,n)*aa(5,n).lt.bv_lim.and.x(n).lt.xbv_lim.and.
     *    ires.eq.0) then
          n1=n-1
          ires=1
        else if(ires.eq.1.and.
     *    (aa(2,n)*aa(5,n).ge.bv_lim.or.x(n).ge.xbv_lim)) then
c
c  test whether interval for resetting is excessive
c
          if(n-n1.ge.kbv_lim) then
            write(istdpr,
     *        '(/'' **** Interval for bv reset is large, '',
     *        ''n1, n ='',2i6/
     *            ''      Do not reset'')') n1, n
              ires=0
          else
            if(ihead.eq.1) write(istdpr,'(//
     *        '' Reset of A_4 to correct buoyancy frequency''/
     *        '' n, x, old A_4, reset A_4, N^2'')')
            ihead=0
c
c  Interpolate in buoyancy frequency to make sure that it never
c  exceeds limit
c
            bvn1=min(aa(5,n1),0.d0)*aa(2,n1)
            bvn=min(aa(5,n),0.d0)*aa(2,n)
            do k=n1+1,n
c
c  at centre, reset from expansion (added 6/6/17)
c
              if(n1.eq.1) then
                aa4_new=aa(5,n)*(x(k)/x(n))**2
              else
                fct1=(x(n)-x(k))/(x(n)-x(n1))
                aa4_new=(fct1*bvn1+(1.d0-fct1)*bvn)/aa(2,k)
                aa4_new=min(0.d0,aa4_new)
              end if
              write(istdpr,'(i6,1p4e13.5)') k, x(k), aa(5,k), aa4_new,
     *        aa(2,k)*aa4_new
              aa(5,k)=aa4_new
            end do
            ires=0
          end if
        end if
      end do
c
      if(aa(5,nn).le.10) then 
        nn=nn-1 
        write(istdou,*) 'Chop off outermost point' 
        if(istdpr.gt.0.and.istdpr.ne.istdou) 
     *    write(istdpr,*) 'Chop off outermost point' 
      end if
c
      data(1)=glob(1)
      data(2)=glob(2)
      data(3)=var(4,1)
      data(4)=var(5,1)
c
      if(glob(11).lt.0.and.glob(11).gt.-10000) then 
        data(5)=-glob(11)/var(10,1)
	data(6)=-glob(12) 
      else 
        data(5)=pi4/3.d0*cgrav*(var(5,1)*glob(2))**2/
     *          (var(4,1)*var(10,1))
	d2amax=0.
	do n=2,5
	  d2amax=max(d2amax,aa(5,n)/x(n)**2)
        end do
   55   data(6)=d2amax+data(5)
	if(istdpr.gt.0) write(istdpr,140) data(5), data(6)
      end if
c
      data(7)=-1.d0
      data(8)=0.d0
c
      nmodel=1
      write(3) nmodel,nn,(data(i),i=1,8),((aa(i,n),i=1,6),n=1,nn)
      stop
  110 format(/' GONG model header:'/)
  120 format(4i10)
  130 format(5e16.9)
  140 format(/' data(5), data(6) reset to ',1p2e13.5)
      end
