      program main
c
c  set standard formatted (fobs) file of oscillation output 
c  from grand summary
c  the type of dataset is determined by the parameter icasin.
c
c  Double precision version
c  ++++++++++++++++++++++++
c
c  Modified 15/11/19, to include information about truncation in the
c  header, when using grand summary.
c
      implicit double precision (a-h, o-z)
      character*80 fin, fout
      logical noprm
      dimension cs(50), ics(12)
      common/c_cgrav/ cgrav
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
      equivalence(cs(39),ics(1))
c
c  icasin: defines type of modes used for input, and controls output.
c  icasin = icase0 + 100*icase2
c  icase0 = 1: grand summary, variational frequency.
c  icase0 = 4: grand summary, from eigenfrequency in cs(20).
c              Note that this allows setting Cowling 
c              approximation frequency.
c  icase0 = 5: grand summary, from Richardson extrapolation frequency 
c              in cs(37), if this is set. 
c              Otherwise variational frequency is used.
c  icase0 = 6: grand summary, from (possibly corrected) eigenfrequency 
c              in cs(21).
c 
c  If icase1 gt 0 include mode energy
c  If icase2 gt 0, and rotational solution is included, print 
c  m values separately, otherwise print only mode with m = 0.
c
      icasin=1
c
      if(istdpr.gt.0) 
     *  write(istdpr,*) 'Enter case number (1, 4, 5 or 6)'
      read(5,*) icasin
      icase0=mod(icasin,10)
      icase2=mod(icasin/100,10)
      write(6,*) 'icase0, icase1, icase2:', icase0, icase1, icase2
c
      if(istdpr.gt.0) 
     *  write(istdpr,*) 'Enter input file name'
      read(5,'(a)') fin
      if(istdpr.gt.0) 
     *  write(istdpr,*) 'Enter output file name'
      read(5,'(a)') fout
c
      open(2, file=fin, status='old',
     *  form='unformatted')
      open(10, file=fout, status='unknown')
      if(istdpr.gt.0) write(istdpr,*) 
     *   'Possibly enter gravitational constant in units of 1.d-8'
      read(5,*,end=10) cgrav
      cgrav=1.d-8*cgrav
      if(istdpr.gt.0) write(istdpr,'(/
     *  '' Use cgrav ='',1pe15.7)') cgrav
c
   10 if(istdpr.gt.0) then
        write(istdpr,*) 'Input file:'
        write(istdpr,*) fin
        write(istdpr,*) 'Output file:'
        write(istdpr,*) fout
      end if
c
      ihead=1
      icasew=icase0
      nordp=-1
      nordg=-1
c
c  start stepping through dataset
c
      nrd=0
      ierrri=0
c
   30 call rdfreq(icase0,2,cs,l,nord,sig,frq,ekin,ierr)
c
      if(ierr.gt.0) go to 60
c
c  test for unavailability of Richardson frequencies
c
      if(ierrri.eq.0.and.ierr.eq.-1.and.icasew.eq.5) then
	ierrri=1
	if(cs(27).gt.0) then
          icasew=1
        else
          icasew=4
        end if
      end if
c
c  test for inclusion of individual m values
c
      noprm=.true.
c
      icase=ics(5)
      irotsl=mod(icase/100,10)
c..	write(6,*) 'icase, irotsl',  icase, irotsl
c
c  with rotational splitting, and icase2 = 0, only print m = 0 modes
c
      if(irotsl.eq.1) then
        if(icase2.eq.1) then
          noprm=.false.
          m=nint(cs(38))
        else if(cs(38).ne.0) then
          go to 30
        end if
      end if
c
c  output header
c
      if(ihead.eq.1) then
        ihead=0
        write(10,'(''# '',a)') fin(1:length(fin))
c
c  test for truncated model
c
        if(cs(12).gt.0) then
          write(10,'(''# Model truncated in interior at r/R ='',
     *      1pe13.5)') cs(12)
        else if(cs(12).lt.0) then
          write(10,'(''# Model truncated in atmosphere at r/R ='',
     *      f12.7)') -cs(12)
        endif
        if(icasew.eq.1) then
          write(10,'(''# using variational frequencies'')')
        else if(icasew.eq.4) then
          write(10,'(''# using (uncorrected) eigenfrequencies'')')
        else if(icasew.eq.5) then
          write(10,'(''# using Richardson frequencies'')')
        else if(icasew.eq.6) then
          write(10,'(''# using (corrected) eigenfrequencies'')')
        end if
c
c  decode mesh flags
c
        data8=cs(9)
        idata8=idint(data8+0.1)
        r_data8=data8-idata8
        if(r_data8.gt.0.and.istdpr.gt.0) then
          i_codecase=idint(1.d3*r_data8+0.1)
          i_meshcase=idint(1.d3*(1.d3*r_data8-i_codecase)+0.1)
          write(10,'(''#''/''# mesh code case: '',i4)') i_codecase
          if(i_meshcase.gt.0)
     *      write(10,'(''# mesh data case: '',i4)') i_meshcase
        end if
c
        write(10,'(''#'')')
        write(10,'(''# Set frequencies with G ='',1pe15.7)') cgrav
        write(10,'(''#'')')
        if(noprm) then
          write(10,'(
     *      ''# l, order, nu (microHz), inertia, nordp, nordg'')')
        else
          write(10,'(
     *      ''# l, m, order, nu (microHz), inertia, nordp, nordg'')')
        end if
        write(10,'(''#'')')
      end if
c
      nrd=nrd+1
c
      nordp=ics(9)
      nordg=ics(10)
c
      if(noprm) then
        write(10,'(2i8,1p2e16.8,2i8)') l, nord, frq, ekin,
     *    nordp, nordg
      else
        write(10,'(3i8,1p2e16.8,2i8)') l, m, nord, frq, ekin,
     *    nordp, nordg
      end if
c
      go to 30
c
   60 continue
      stop
      end
