      subroutine filt_obs(frq,i_frq,n_frq,k_obsfit,obsfrq,i_obsfrq,
     *  n_obsfrq,frq_filt,i_filt,n_filt,idiag)
c
c  Eliminates mixed modes by selecting one mode per radial Delta nu
c  interval, choosing the mode with the lowest inertia.
c
c  If k_obsfit gt 0 include distance to observed frequencies as 
c  part of the selection criterion for l = 1.
c  The observed frequencies must be provided in obsfrq.
c
c  Original version: 8/1/15
c
c  Modified 10/2/16 to include observed frequencies in selection
c
      implicit double precision (a-h, o-z)
      logical test_obs
      parameter(nmax=10000,iw=10,i_obsmod=10,ih=5)
      dimension frq(i_frq,*), obsfrq(i_obsfrq,*), frq_filt(i_filt,*) 
      dimension w(iw,nmax),frq_0(nmax),ord_0(nmax),d_obs(nmax),
     *  obsfrq1(4,nmax),np0(nmax),obsfit(3),n_orig(nmax)
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
      save
c
c  hard-code weight given to departure from observed frequencies
c
      w_obs=5.d0
c
c  set scaled mode inertia
c
      nn0=0
      do n=1,n_frq
        w(2,n)=dlog(frq(4,n))
        if(frq(1,n).eq.0.d0) then
          nn0=n
          frq_0(n)=frq(3,n)
          ord_0(n)=frq(2,n)
          w(4,n)=w(2,n)
          w(3,n)=1.d0
          w(5,n)=1.d0
          np0(n)=frq(2,n)
          w(7,n)=frq(2,n)
        else
          call lir1(frq(3,n),frq_0,w(4,n),w(2,1),1,iw,nn0,1,
     *      inter)
          w(3,n)=dexp(w(2,n)-w(4,n))
        end if
      end do
c
c  select observed modes with l = 1
c
      n1=0
      do n=1,n_obsfrq
        if(obsfrq(1,n).eq.1.d0) then
          n1=n1+1
          do i=1,4
            obsfrq1(i,n1)=obsfrq(i,n)
          end do
c..          write(6,*) 'n, n1, obsfrq1(3,n1)',n, n1, obsfrq1(3,n1)
        end if
      end do
c
      n_obsfrq1=n1
      obs1_min=obsfrq1(3,1)
      obs1_max=obsfrq1(3,n_obsfrq1)
c
c  set np0 such that it flags the order of the radial mode nu_0,
c  for which the given frequency is in the interval
c  [nu_np0,0,nu_np0+1,0]
c  Also eliminate modes below initial mode (typically lowest-order
c  radial mode)
c
      frqmin0=frq(3,1)
      frqmax0=frq(3,nn0)
      l_test=frq(1,n_frq)+1
c
      do n=n_frq,nn0+1,-1
        if(frq(3,n).lt.frqmin0) then
          np0(n)=0
          w(5,n)=-1
        else if(frq(3,n).ge.frqmax0) then
            w(5,n)=1
            np0(n)=ord_0(nn0)
        else
          w(5,n)=-1
          test_obs=frq(1,n).eq.1.and.k_obsfit.eq.1.and.
     *             obs1_min.le.frq(3,n).and.frq(3,n).le.obs1_max
          if(test_obs) then
            w(6,n)=obs_dist(frq(1,n),obsfrq1,4,n_obsfrq1,
     *        obsfit)
c..            write(6,*) 'nu, obs_dist', frq(3,n),w(6,n)
          else
            w(6,n)=0.d0
          end if
c
c  test for new degree
c
          if(frq(1,n).lt.l_test) then
            l_test=frq(1,n)
            n0=nn0-1
            frq0=frq(3,n0)
          end if
c
c  set radial interval
c
          if(frq(3,n).lt.frq0) then
            n0=n0-1
            frq0=frq(3,n0)
          end if
          np0(n)=frq(2,n0)
        end if
      end do
c
c  renormalize departure from observations and add to selection parameter
c
      if(k_obsfit.eq.1) then
        l_test=frq(1,n_frq)+1
        nc0=-1
        do n=n_frq,nn0+1,-1
          if(frq(1,n).eq.1) then
            if(nc0.eq.-1) then
              nc0=np0(n)
              if(w(6,n).gt.0) then
                d_sum=w(6,n)
              else
                d_sum=1.d0
              end if
              ns=n
              i=1
c..              write(6,*) 'i, w(6,n), d_sum',i,ns,n,w(6,n),d_sum
            else if(np0(n).eq.nc0) then
              i=i+1
              d_sum=d_sum+w(6,n)
c..              write(6,*) 'i, w(6,n), d_sum',i,ns,n,w(6,n),d_sum
            else
              itot=i
              if(itot.gt.1.and.frq(3,n).ge.obs1_min.and.
     *          frq(3,n).le.obs1_max) then
                do i=1,itot
                  n1=ns-i+1
c..                  write(6,'(i5,f4.1,f10.3,1p3e13.5,i4)') 
c..     *              n1,frq(1,n1),frq(3,n1),w(3,n1),
c..     *              w_obs*w(6,n1)/d_sum,
c..     *              w(3,n1)+w_obs*w(6,n1)/d_sum,nc0
                  w(3,n1)=w(3,n1)+w_obs*w(6,n1)/d_sum
                end do
              end if
              nc0=np0(n)
              if(w(6,n).gt.0) then
                d_sum=w(6,n)
              else
                d_sum=1.d0
              end if
              ns=n
              i=1
c..              write(6,*) 'i, w(6,n), d_sum',i,ns,n,w(6,n),d_sum
            end if
          end if
        end do
      end if
c
c..      write(6,'(/'' l, n, nu, E, np(0):''/(0p2f6.1,f10.3,1pe13.5,i4))')
c..     *  ((frq(i,n),i=1,4),np0(n),n=1,n_frq)
c
c  find minimum of parameter in each interval
c
      l_test=frq(1,n_frq)+1
c
      do n=n_frq,nn0+1,-1
        if(frq(1,n).lt.l_test) then
          nc0=np0(n)
          wmin=w(3,n)
          nmin=n
c..          write(6,'(i5,2f5.1,f10.3,1pe13.5,i4)') 
c..     *      n, (frq(i,n),i=1,3),w(3,n),nc0
          w(7,n)=nc0-int((frq(1,n)-1)/2)
          l_test=frq(1,n)
        else if(np0(n).eq.nc0) then
          if(w(3,n).lt.wmin) then
            nmin=n
            wmin=w(3,n)
          end if
c..          write(6,'(i5,2f5.1,f10.3,1pe13.5,i4)') 
c..     *      n, (frq(i,n),i=1,3),w(3,n),nc0
          w(7,n)=nc0-int((frq(1,n)-1)/2)
c
c  new radial-order interval
c
        else
          w(5,nmin)=1
c..          write(6,'(i5,2f5.1,f10.3,f5.1)') 
c..     *      nmin, (frq(i,nmin),i=1,3),w(5,nmin)
          nc0=np0(n)
          wmin=w(3,n)
          nmin=n
          w(7,n)=nc0-int((frq(1,n)-1)/2)
c..          write(6,'(i5,2f5.1,f10.3,1pe13.5,i4)') 
c..     *      n, (frq(i,n),i=1,3),w(3,n),nc0
        end if
      end do
c
c  store filtered frequencies in frq_filt
c
      n1=0
      do n=1,n_frq
        if(w(5,n).gt.0) then
          n1=n1+1
          do k=1,4
            frq_filt(k,n1)=frq(k,n)
          end do
          n_orig(n1)=n
        end if
      end do
c
      n_filt=n1
c
c  reset order, assumed to be correct at highest frequency for
c  each degree
c
      l_test=frq(1,n_frq)+1
      do n=n_filt,1,-1
        if(frq_filt(1,n).lt.l_test) then
          l_test=frq_filt(1,n)
        else
          frq_filt(2,n)=frq_filt(2,n+1)-1
        end if
        if(frq_filt(2,n).ne.w(7,n_orig(n))) then
          if(istdpr.gt.0) 
     *      write(istdpr,'('' order problems'',3f5.1,f10.3)')
     *      frq_filt(1,n),frq_filt(2,n),w(7,n_orig(n)),frq_filt(3,n)
          frq_filt(2,n)=w(7,n_orig(n))
        end if
      end do
c
      if(idiag.ge.1.and.istdpr.gt.0) 
     *  write(istdpr,'(/'' Filtered modes.  l, n, freq, E:''/
     *    (0p2f5.1,f10.3,1pe13.5))') ((frq_filt(k,n),k=1,4),n=1,n_filt)
      return
      end
      function obs_dist(frq,obsfrq,i_obsfrq,n_obsfrq,obsfit)
c
c  determine closest-fitting observed mode to model data in frq
c  (assumed to give l, n, frequency)
c
      implicit double precision (a-h, o-z)
      dimension frq(*),obsfrq(i_obsfrq,*),obsfit(*)
c
      nmin=0
      do n=1,n_obsfrq
        if(obsfrq(1,n).eq.frq(1)) then
          dd=(obsfrq(3,n)-frq(3))/obsfrq(4,n)
          dd=dd*dd
          if(nmin.eq.0) then
            nmin=n
            ddmin=dd
          else if(dd.lt.ddmin) then
            nmin=n
            ddmin=dd
          end if
        end if
      end do
c
      do i=1,3
        obsfit(i)=obsfrq(i,nmin)
      end do
c
      obs_dist=ddmin
      return
      end
