      subroutine dump_obs(obs_file_in,iagsm)
c
c  dumps mode quantities in common/cobs_param/.
c  If iagsm .le. 0, write to file obs_file_in.
c  Else set file name from name of agsm file, assumed to be assigned
c  to unit iagsm, replacing 'agsm' by 'obs'.
c
c  The frequency is set, depending on ispcpr and mdintg:
c
c  If mdintg = 2 or 5, use (possibly corrected) eigenfrequency, 
c              regardless of value of ispcpr.
c
c  ispcpr = 1: variational frequency.
c  ispcpr = 4: from eigenfrequency in cs(20).
c              Note that this allows setting Cowling
c              approximation frequency.
c  ispcpr = 5: from Richardson extrapolation frequency
c              in cs(37), if this is set.
c              Otherwise variational frequency is used.
c  ispcpr = 6: from (possibly corrected) eigenfrequency
c              in cs(21).
c
c  If ispcpr = 0, the frequency is set in order of priority, depending
c  on the availability of the relevant quantity: 5, 1, 6
c  The same order of priority is used if the quantitity selected by
c  another value of ispcpr is not available.
c
c  The actual quantity returned is flagged by icobs_st, set when 
c  nobs_st = 0 (and assumed valid for the full set for the given
c  model).
c
c  original version 8/7/05
c
c  Modified 3/8/05, allowing for rotational results
c
c  Modified 28/3/14, including option of setting file name from 
c  agsm file.
c
c  Modified 2/6/15, increasing character length of file and filess
c
      implicit double precision (a-h,o-z)
      include 'adipls.c.d.incl'
      character obs_file_in*(*)
      character*256 strcompr, str_replace, obs_file, file, filess,
     *  trailer_par, msh_trailer, omega_file
c
c  common for storage of modal parameters 
c  degree, order, cyclic frequency (microHz), inertia
c
      common/trl_param/ trailer_par
      common/cmsh_param/ msh_trailer
      common/cincnt_new/ fsig0, mdintg
      common/cobs_param/ icobs_st, nobs_st, obs_st(10,nobs_stmax)
      common/cofile/ nfiles, idsfil(99), file(99), iopen(99), filess(99)
      common/c_omega_file/ omega_file
c
c  common defining standard input and output, standard error
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
c  test for no data
c
c..      write(istder,*) '#D# Enter dump_obs with icobs_st =',icobs_st
c..      write(istdou,*) '#D# Enter dump_obs with icobs_st =',icobs_st
c
      if(nobs_st.le.0) then
        write(istdou,105)
        if(istdpr.gt.0.and.istdpr.ne.istdou) write(istdpr,105)
        return
      end if
c
      if(iagsm.le.0) then
        obs_file=obs_file_in
      else
c
c  set output file name from corresponding agsm file
c
        call stfile(iagsm,nfsagsm)
        obs_file=str_replace(file(nfsagsm),'agsm','obs',ierr)
        obs_file=strcompr(obs_file//trailer_par//'.'//msh_trailer)
        if(istdpr.gt.0) write(istdpr,'(//
     *    '' In dump_obs obs_file set to '',a)')
     *    obs_file(1:length(obs_file))
      end if
c
      open(99,status='unknown',file=obs_file,form='formatted')
      write(99,110)
      icobs_st0=mod(icobs_st,10)
      icobs_st1=icobs_st/10
      if(icobs_st0 .eq. 1) then
        write(99,120) 'Variational frequency'
      else if(icobs_st0 .eq. 4) then
        write(99,120) 'Raw eigenfrequency'
      else if(icobs_st0 .eq. 5) then
        write(99,120) 'Richardson extrapolation frequency'
      else 
        write(99,120) 'Eigenfrequency (possibly corrected)'
      end if
c
      if(icobs_st1.eq.0) then
	write(99,125)
	imax=4
      else if(icobs_st1.gt.1) then
        l_omega_file=length(omega_file)
        write(99,'(''# Rotation from file: '',a/''#'')')
     *    omega_file(1:l_omega_file)
         write(99,129)
         imax=8
      else
        l_omega_file=length(omega_file)
        write(99,'(''# Rotation from file: '',a/''#'')')
     *    omega_file(1:l_omega_file)
	write(99,127)
	imax=6
      end if
c
      do n=1,nobs_st
        ll=intgpt(obs_st(1,n)+0.5)
        nord=intgpt(obs_st(2,n)+0.5)
        write(99,130) ll, nord, (obs_st(i,n),i=3,imax)
      end do
      close(99)
c
      return
  105 format(//' ***** Error in s/r dump_obs. No data stored'/)
  110 format('#  Dump contents of common/cobs_param/'/'#')
  120 format('#  Frequency from: ',a/'#')
  125 format('# degree, order, frequency (microHz), inertia'/'#')
  127 format(
     *   '# First-order rotation expansion;'/
     *   '# Frequency is calculated as'/
     *   '# nu_nlm = frequency + m*splitting1'/
     *   '#  '/
     *   '# degree, order, frequency (microHz), inertia, ',
     *   ' beta, splitting1 (microHz)'/'#')
  129 format(
     *   '# Second-order rotation expansion;'/
     *   '# splitting(1) - splitting(3) correspond to omega_Omega1,'/
     *   '# Delta_nl((4) and Delta_nl^(5) in Kara Burke notes.'/
     *   '# Thus the frequency is calculated as'/
     *   '# nu_nlm = frequency + m*splitting1 + splitting2 '/
     *   '#          + m^2*splitting3'/
     *   '#  '/
     *   '# degree, order, frequency (microHz), inertia, ',
     *   ' beta, splitting1 (microHz), splitting2 (microHz), ',
     *   ' splitting3 (microHz) '/'#')
  130 format(i5,i6,f11.4,1pe13.5,0pf10.6,1pe13.5,1pe13.5,1pe13.5)
      end
