      subroutine kiner_sep(x,y,aa,el,sig,iy,ia,nn,ekin_tot,ekin_core)
c
c  calculates kinetic energy integrals for core and total integral
c  Unlike s/r kiner, this uses trapezoidal integration for stable
c  integration over g-mode propagation regions (where sigma < N)
c
c  Double precision version.
c  +++++++++++++++++++++++++
c
c  Original version: 3/11/15
c
c  Normalize energy by squared norm of total displacement,
c  evaluated at photosphere, for iekinr = 1
c  For iekinr = 0 normalization with surface vertical displacement
c  is used. iekinr is transmitted in common/cincnt/.
c
      implicit double precision (a-h,o-z)
      include 'adipls.c.d.incl'
      dimension x(nn),y(iy,nn),aa(ia,nn)
      dimension yi(2)
      common/worksp/ wrk(3,nnmax)
      common/cincnt/ xmnevn,xfit, fcnorm, eps, epssol, dsigmx, fsig, 
     *  dsigre,icow,iturpr,iplneq,iriche,nnwwin, moddet, itmax, irsevn, 
     *  nftmax,irsord,iekinr,itsord,imissl,imjssl,imstsl
      common/cprcns/ epsprc,epsufl,epsofl,eprufl
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      save
c
      pi = 4.d0*atan(1.d0)
c
c  set integrand
c  test for plane-parallel case
      if(iplneq.eq.1) then
        elli=1.d0/(el*el)
        do 5 n=1,nn
        y1=y(1,n)
        y2=y(2,n)
c  avoid underflows
        if(abs(y1).lt.eprufl) y1=0
        if(abs(y2).lt.eprufl) y2=0
    5   wrk(1,n)=(y1*y1+elli*y2*y2)*aa(5,n)
c
      else if(el.gt.1.e-6) then
c
c  nonradial case
c
        elli=1.d0/(el*(el+1))
        do 10 n=1,nn
        y1=y(1,n)
        y2=y(2,n)
c  avoid underflows
        if(abs(y1).lt.eprufl) y1=0
        if(abs(y2).lt.eprufl) y2=0
        xx=x(n)
   10   wrk(1,n)=(y1*y1+y2*y2*elli)*aa(1,n)*aa(5,n)*aa(10,n)*xx*xx
c
      else
c
c  radial case
c
        do 25 n=1,nn
        xx=x(n)
        y1=y(1,n)
   25   wrk(1,n)=y1*y1*aa(1,n)*aa(5,n)*aa(10,n)*xx*xx
c
      end if
c
c  integrate over whole star
c
      ekin_tot=0.d0
      do n=2,nn
        ekin_tot=ekin_tot+0.5d0*(x(n)-x(n-1))*(wrk(1,n-1)+wrk(1,n))
      end do
c
      ekin_core=0.d0
c
      if(el.le.1.d-6) go to 30
c
c  integrate over g-mode regions
c
c  set squared dimensionless buoyancy and acoustic frequency 
c
      ell=el*(el+1.d0)
      do n=1,nn
        wrk(2,n)=aa(1,n)*aa(4,n)
        wrk(3,n)=ell*aa(1,n)/(aa(2,n)+1.d-20)
      end do
      do n=2,nn
        if(wrk(2,n).ge.sig.and.wrk(3,n).ge.sig) then
          if(wrk(2,n-1).lt.sig.and.wrk(3,n-1).ge.sig) then
            xp=x(n)+(wrk(2,n)-sig)*(x(n)-x(n-1))/(wrk(2,n-1)-wrk(2,n))
            wrkp=wrk(1,n)+(wrk(1,n-1)-wrk(1,n))*(xp-x(n))/(x(n-1)-x(n))
          else
            xp=x(n-1)
            wrkp=wrk(1,n-1)
          end if
          ekin_core=ekin_core+0.5d0*(x(n)-xp)*(wrkp+wrk(1,n))
        else
          if(wrk(2,n-1).ge.sig.and.wrk(3,n-1).ge.sig) then
            xn=x(n)+(wrk(2,n)-sig)*(x(n)-x(n-1))/(wrk(2,n-1)-wrk(2,n))
            wrkn=wrk(1,n)+(wrk(1,n-1)-wrk(1,n))*(xn-x(n))/(x(n-1)-x(n))
            ekin_core=ekin_core+0.5d0*(xn-x(n-1))*(wrk(1,n-1)+wrkn)
          end if
        end if
      end do
c
   30 continue
c
c  normalize result, depending on iekinr
c
      if(iekinr.ne.1) then
        if(y1.eq.0) y1=1
	ynorm=y1*y1
      else
	call lir(1.d0,x,yi,y,2,iy,nn,1,inter)
	if(el.le.1.e-6) then
	  ynorm=yi(1)*yi(1)
        else
	  ynorm=yi(1)*yi(1)+elli*yi(2)*yi(2)
        end if
      end if
      ekin_tot=ekin_tot/(4.d0*pi*ynorm)
      ekin_core=ekin_core/(4.d0*pi*ynorm)
      return
      end
c
c
