      subroutine set_xfit(el,x,data,aa,ia,nn,nibc,xfit_in,nev1,
     *  xfit_rad,xfit,nfit,call_routine)
c  Sets fitting point xfit and nfit depending on xfit_in
c
c  For xfit_in.ge.0, no action
c
c  For -1.lt.xfit_in.le.0 set fitting point just outside boundary of
c  evanescent region, if outside abs(xfit_in))
c  Otherwise use abs(xfit_in)
c  For xfit_in = -2, set fitting point at first local maximum in 
c  buoyancy frequency, for el .gt. 0.
c  Features immediately outside a convective core, and very sharp
c  features, are ignored.
c  For radial modes, fitting point is set to xfit_rad, provided as
c  input to routine.
c
c  Original version: 17/12/18.
c
c  Modified 4/6/19, with option to set xfit in relation 
c  to boundary of evanescent region
c
      implicit double precision(a-h, o-z)
      character*(*) call_routine
      dimension x(*), data(*), aa(ia,*)
      common/cincnt/ xmnevn,xfitcom, fcnorm, eps, epssol, dsigmx, fsig,
     *  dsigre,icow,iturpr,iplneq,iriche,nnwwin, moddet, itmax, irsevn,
     *  nftmax,irsord,iekinr,itsord,imissl,imjssl,imstsl,inomd1,itrsg1
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      if(istdpr.gt.0) write(istdpr,
     *  '(/'' Entering set_xfit from '',a/
     *     ''  xfit, nev1, x(nev1) ='',1pe13.5,i7,e13.5)')
     *    call_routine,xfit_in, nev1, x(nev1)
c
c  for xfit_in.ge.0 do not reset xfit
c
      if(xfit_in.ge.0) then
        xfit=xfit_in
        if(istdpr.gt.0) write(istdpr,'('' Exit set_xfit'')')
        return
      end if
c
c  for radial modes, fit close to surface
c
      if(el.le.1.d-6) then
        xfit=xfit_rad
        if(istdpr.gt.0) write(istdpr,'('' xfit set to'',1pe13.5,
     *    '' from input to routine'')') xfit_rad
        n=0
        do while(x(n).lt.xfit)
          nfit=n
          n=n+1
        end do
c
      else if(-1.d0.lt.xfit_in) then
        xfit=-xfit_in
        xev1=x(nev1)
        xfit_lim=min(1.01*xev1,0.98*xev1+0.02)
        if(xfit.lt.xfit_lim) xfit=xfit_lim
        n=0
        do while(x(n).lt.xfit)
          nfit=n
          n=n+1
        end do
c
      else if(xfit_in.eq.-2) then
c
        xfit=0
        n=1
        xcc=0
        bvp=0
c
        do while(xfit.eq.0.and.n.le.nn) 
          n=n+1
          bv=aa(1,n)*aa(4,n)
c..        write(6,*) 'n,x,bv:',n,x(n),bv
          if(bv.le.0) then
            xcc=x(n)
          else if(x(n).ge.1.2*xcc) then
            if(bv.lt.bvp.and.bv.ge.0.95*bvp) then
              xfit=x(n)
              nfit=n
            end if
          end if
          bvp=bv
        end do
        if(istdpr.gt.0) write(istdpr,'(/
     *    '' In set_xfit with input -2, xfit set to'',1pe13.5)') xfit
      end if
c
c  test that nfit - nibc is even, for Richardson extrapolation
c
      if(iriche.eq.1) then
        if(mod(nfit-nibc,2).eq.1) then
          xfit_old=xfit
          nfit=nfit+1
          xfit=x(nfit)
          if(istdpr.gt.0) write(istdpr,'(/
     *      '' For Richardson extrapolation,''/
     *      '' nfit, xfit reset from'', i5,1pe13.5,'' to'',i5,e13.5)')
     *      nfit-1,xfit_old,nfit,xfit
        end if
      end if
      if(istdpr.gt.0) write(istdpr,'(/'' Exit set_xfit'')')
      return
      end
