      subroutine sigitr(sig,x,y,iy,nw1,nibc,nn,
     *  mdintg,nev1,iasn,ii,ig,ifrich,nfit,istsb1,iscan,itrsig,
     *  xfit1,det,ddsig,ddsol,ds1,iord,icry,isolcv,isigcv,istatus)
c
c  Iterates to determine sig = sigma**2 by integrating
c  adiabatic oscillation equations for given l = el.
c  (transmitted in common/rhsdat/). Integration method is determined
c  by mdintg.
c
c  ifrich flags for Richardson extrapolation
c  ifrich = 0: no Richardson extrapolation
c  ifrich = 1: called on thinned mesh with Richardson extrapolation
c  ifrich = 2: called on full mesh with Richardson extrapolation
c
c  Returns matching condition in det, absolute value of last 
c  relative change in sig in ddsig, possibly test of eigenfunction
c  continuity in ddsol and matching determinant, to be used in
c  s/r mchsol, in ds1 (ddsol and ds1 are not set for mdintg = 3).
c
c  Returns isigcv = 1 if eigenfrequency is converged, otherwise -1
c  Returns isolcv = 1 if eigenfunction is converged, otherwise -1
c
c  icry returns error flag:
c  icry = -1: error in integration in sigint
c  icry = -5: singularity in nrkint with mdintg = 3.
c  icry = -6: Solution deviates from initial value by more than
c  than 5*dsig_init. 
c  is initial step in sig, return to sigitc with icry = -6.
c
c  Original version 30/6/95
c
c  Modified 22/10/96, to set flag for no convergence if det eq detp.
c
c  Modified 11/6/19, changing sign of first step if the initial change
c  in the fitting determinant is too small.
c  Note: this is not (currently) employed on full mesh with 
c  Richardson extrapolation. May need further investigation.
c
c  Modified 12/6/19, introducing an internal scan over the interval
c  sigi2, sigi1 when sigitr is call from a frequency search based on
c  a scan. This is activated if the iteration results in a value of sig
c  outside [sigi2, sigi1].
c  Note: this is not employed on full mesh with Richardson extrapolation
c  since scan takes place on thinned mesh.
c
c  Modified 22/8/19, correcting test for being close to a maximum in 
c  det (previously, no upper limit was given).
c
c  Modified 9/6/21, updating error flag icry, introducing icry = -5
c  for nrkint singularity.
c
c  Modified 11/3/22, initializing isigcv, isolcv to -1 on entry to
c  routine
c  When called with iscan .le. 1, introduce test on departure of sig
c  from initial value. If greater than 5*dsig_init, where dsig_init
c  is initial step in sig, return to sigitc with icry = -6.
c
c  Modified 25/3/22, suppressing call of stevft before iteration
c  if mdintg = 3 and ifine_nrk > 0 (i.e., during fine scan)
c
c  Modified 29/3/22, extend range of internal scan below and above
c  input range.
c
      implicit double precision(a-h, o-z)
      logical fulmod, radial
      include 'adipls.c.d.incl'
      parameter(iaa=10)
      dimension x(*), y(iy,*), ds1(4,*)
c
      common/rhsdat/ el,ell,alb,els,el1,sigcom,anres,perfac,data(8),
     *  aa(iaa,nnmax)
      common/cincnt/ xmnevn,xfit, fcnorm, eps, epssol, dsigmx, fsig, 
     *  dsigre,icow,iturpr,iplneq,iriche,nnwwin, moddet, itmax, irsevn, 
     *  nftmax,irsord,iekinr,itsord,imissl,imjssl,imstsl
      common/cprcns/ epsprc,epsufl,epsofl,eprufl
      common/c_scanctrl/ sigi1, sigi2
      common/c_nrkdiag/  signrk_p, signrk, det_norm_p, det_norm,
     *  isig_nrk, ising_nrk, ifine_nrk
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      if(istdpr.gt.0) write(istdpr,'(/'' Enter sigitr'')')
c
      sig_init=sig
      ifine=0
c
c  set flag for radial oscillations
c
      radial=el.le.1.e-6
      fulmod=nibc.eq.2
c
c  flags for convergence
c
      isolcv=0
      isigcv=0
c
c  initialize det and detp to zero
c
   10 det=0
      detp=0
c
c  take out factor x**l in solution up to point nev1 where solution
c  begins to vary more slowly than x**(l/2), or to nfit if
c  nfit .le. nev1.
c
c  For consistency with old version of programme, do not reset
c  boundary for iteration during scan, if irsevn = 0
c
c  25/3/22: suppress for mdintg = 3, ifine .ge.1
c
      if(ifrich.ne.2.and.(iscan.le.1.or.irsevn.ge.1).and.
     *  (mdintg.ne.3.or.ifine_nrk.eq.0)) then
        els=ell/sig
        call stevft(x,nn,nibc,nfit,iasn,radial,fulmod,nev1,xfit1,
     *    'sigitr')
c
c  possibly reset fitting point 
c
        call set_xfit(el,x,data,aa,iaa,nn,nibc,xfit,nev1,0.95d0,
     *    xfit1,nfit,'sigitr')
        if(istdpr.gt.0.and.xfit1.ne.xfit) write(istdpr,'(/
     *    '' xfit, x(nfit), nfit set to'',2f10.6,i6,'' in sigitr'')')
     *  xfit1, x(nfit), nfit
      end if
c
c  set flag for initialization in nrkint (only used for mdintg=3)
c
      initnr=1
c
      isig=-1
c
c  begin integration with new sig
c  ******************************
c
   14 isig=isig+1
      detpp=detp
      detp=det
      els=ell/sig
      sigcom=sig
c
c  carry out integration for given sig and l
c
      call sigint(x,y,iy,nw1,nibc,nn,mdintg,nev1,iasn,ii,ig,
     *  initnr,nfit,istsb1,isig,det,ddsol,ds1,iord,icry)
c
c  test for error in integration
c
      if(icry.lt.0) then
        if(istdpr.gt.0) write(istdpr,'(/'' Exit sigitr with error'')')
        return
      end if
c
c  Test for possible singularity in nrkint
c
      if(mdintg.eq.3.and.ising_nrk.eq.1) then
c
c  Note (23/2/22): For now, suppress action
c  Note (10/3/22) turned on again
c  Note (15/3/22) turned off again
c
        if(istdpr.gt.0) write(istdpr,'(/
     *    '' ***** In sigitr, singularity flagged in nrkint'')')
        if(istdpr.gt.0) write(istdpr,'(
     *    '' Note 15/3/22: for now ignore'')')
c..        icry = -5
c..        return
      end if
c
c  output of matching results
c  **************************
c
      if(mdintg.ne.2) then
c
        if(istdpr.gt.0) write(istdpr,'(
     *        '' isig,sig,det,det/detnrm:'',i4,f17.10,1p2e11.3)')
     *        isig,sig,det,ddsol
      else
c
        if(istdpr.gt.0) write(istdpr,'(
     *    '' isig,sig,det,det/detnrm,order:'',i3,f16.9,1p2e11.3,i8)')
     *    isig,sig,det,ddsol,iord
c
      end if
c
c  for itmax = 0, test continuity of eigenfunction
c
      if(itmax.eq.0) then
        if(ddsol.ge.epssol) then
c
c  solution discontinuous. set flags for no convergence
c
          write(istdou,159)
          if(istdpr.ne.istdou.and.istdpr.gt.0) write(istdpr,159)
          isolcv=-1
          isigcv=-1
        end if
c 
        go to 26
c
      end if
c
c  first step
c
      if(isig.eq.0) then
c
c  set second sigma**2
c
        sigp=sig
        dsig=fsig*sigp
        dsig_init=dsig
        sig=sigp+dsig
        ddsig=abs(dsig/sig)
c..        if(istdpr.gt.0) write(istdpr,'(/
c..     *    '' In sigitr fsig, sigp, dsig, sig =''/1p4e20.12)')
c..     *    fsig, sigp, dsig, sig
        go to 14
c
      end if
c
c  subsequent steps, set change in sig
c
      if(ifrich.ne.2.and.ifine.eq.0.and.isig.eq.1.and.
     *  det/detp.ge.0.7.and.det/detp.le.1.4) then
c
c  close to a maximum. Try negative change
c
        dsig=-fsig*sig_init
        sig=sig_init+dsig
        det=detp
        if(istdpr.gt.0) write(istdpr,'(/
     *    '' In sigitr det close to maximum. det/detp ='',f10.5/
     *    '' Set dsig, sig to'',1p2e20.12)') det/detp, dsig,sig
        go to 14
      end if

c
      if(abs(detp-det).le.epsprc*abs(detp)) then
c
c  no significant change in det. set dsig to 0 to force end of iteration
c  and set flags for no convergence.
c
        write(istdpr,'(/
     *    '' ********** no change in det in s/r adipls.''/
     *    12x,''det, detp ='',1p2e23.15/
     *    12x,''sig, sigp ='',2e23.15)') det,detp,sig,sigp
        dsig=0
        isolcv=-1
        isigcv=-1
      else
c
        dsig_old=dsig
        dsig_new=dsig*det/(detp-det)
        dsig=dsig_new
c
      end if
c
      ddsigp=ddsig
      ddsig=abs(dsig/sig)
c
c  limit ddsig to less than dsigmx
c
c..      write(istdpr,*) '#D# dsig, dsigmx =',dsig, dsigmx
      if(ddsig.ge.dsigmx) then
        dsig=sign(dsigmx,dsig)*abs(sig)
        ddsig=dsigmx
      end if
c
      sig=sig+dsig
c
c  test for solution wandering too far from input
c
      if(iscan.le.1.and.abs(sig-sig_init).gt.5*dsig_init) then
        icry=-6
        if(istdpr.gt.0) write(istdpr,'(/
     *    '' ***** sig = '',1pe13.5,'' too far from sig_init ='',
     *    e13.5/''       Try new iteration''//
     *    '' Exit sigitr'')') sig,sig_init
        return
      end if
c
c  test for convergence
c  ********************
c
      if(ddsig.gt.eps) then
c
c  during scan, test that sig is in the appropriate interval.
c  otherwise make a separate local scan
c
        if(ifrich.ne.2.and.ifine.eq.0.and.iscan.gt.1.and.
     *    (sig.le.sigi2.or.sig.ge.sigi1)) then
          iscloc=11
          dsgloc=(sigi1-sigi2)/(iscloc-1)
c
c  start scan dsgloc before interval and extend dsglog after interval
c  (29/3/22)
c
          sigl=sigi2-2*dsgloc
          siglp=0
          detl=0
          detlp=0
          isigl=0
          if(istdpr.gt.0) write(istdpr,'(/
     *      '' sig ='',1pe15.7,'' is outside interval'',2e15.7/
     *      '' identified by scan.''
     *      '' Start internal scan in s/r sigitr''/)') sig, sigi2, sigi1
          do while(isigl.le.iscloc+2.and.detlp*detl.ge.0)
            isigl=isigl+1
            detlp=detl
            siglp=sigl
            sigl=sigl+dsgloc
            els=ell/sigl
            sigcom=sigl
c
c  carry out integration for given sig and l
c
            call sigint(x,y,iy,nw1,nibc,nn,mdintg,nev1,iasn,ii,ig,
     *        initnr,nfit,istsb1,isig,detl,ddsol,ds1,iord,icry)
            if(istdpr.gt.0) write(istdpr,'(
     *        '' isig,sig,det,det/detnrm:'',i4,f17.10,1p2e11.3)')
     *        isigl,sigl,detl,ddsol
c
c  Test for possible singularity in nrkint or other errors
c
            if(mdintg.eq.3.and.ising_nrk.eq.1) then
              icry = -5
              if(istdpr.gt.0) write(istdpr,'(/
     *          '' ***** In sigitr, singularity flagged in nrkint'')')
            end if
            if(icry.lt.0) then
              if(istdpr.gt.0) write(istdpr,'(/
     *          '' Exit sigitr with error'')')
              return
            end if
          end do
c
c  test for solution on interval sigi2, sigi1
c
          if(detlp*detl.ge.0) then
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' No solution found in interval '',1p2e15.7/
     *        '' Return with error''/)') sigi2, sigi1
            isolcv=-1
            isigcv=-1
            icry=-1
            if(istdpr.gt.0) write(istdpr,'(/'' Exit sigitr'')')
            return
          end if
c
c  restart iteration with result of scan
c
          sig=(detl*siglp-detlp*sigl)/(detl-detlp)
          ifine=1
          fsig=0.1*(sigl-siglp)/siglp
          if(istdpr.gt.0) write(istdpr,'(/'' Restart iteration'')')
          go to 10
        end if
c

c  frequency not converged. try again if maximum number of iterations
c  not exceeded
c
        if(isig.lt.itmax) then
          go to 14
        else
c
c  unconverged. do not file mode and/or write summary
c
          isigcv=-1
          isolcv=-1
          istatus=1
          write(istdou,122) ddsig,eps
          if(istdpr.ne.istdou.and.istdpr.gt.0) write(istdpr,122) 
     *      ddsig,eps
          go to 26
c
        end if
c
      else
c
c  frequency converged. test for convergence of eigenfunction
c
        isigcv=1
        if(ddsol.lt.epssol) then
c
c  everything converged. finish for this mode
c
          isolcv=1
          go to 26
c
        else
c
c  eigenfunction not converged.
c
c  test for convergence of sig to full accuracy. otherwise try again.
c
          if(ddsig.gt.epsprc.and.isig.lt.itmax) then
            go to 14
c
c  give up on eigenfunction
c
          else
c
            write(istdpr,'(/
     *        1x,10(1h*),'' ddsig ='',1pe13.5,'' has converged,'',
     *        '' but ddsol ='',e13.5,'' .gt. epssol ='',e13.5/
     *        11x,'' eigenfunction may be inaccurate'')')
     *        ddsig,ddsol,epssol
            isolcv=-1
            istatus=2
c
c  end of tests for convergence
c
          end if
c
        end if
c
      end if
c
c  end of routine
c
   26 continue
c
      if(istdpr.ne.istdou.and.isigcv.eq.1) write(istdou,185) isig, sig
c
      if(istdpr.gt.0) write(istdpr,'(/'' Exit sigitr'')')
      return
  137 format(/)
  122 format(//1x,10(1h*),'  iteration unconverged. dsig =',
     *  1pe13.5,' .gt.  eps =',e13.5)
  159 format(//' **** solution discontinuous for',
     *  ' itmax = 0')
  185 format(' Iteration converged after ',i3,
     *  ' steps.    sigma**2 =',1pe12.4)
      end
