      subroutine sigscn(sig1,sig2,iscan,nsig,itrsig,x,y,iy,
     *  nw1,nibc,nn,nnw,mdintg,nev1,nfit,istsb1,icry)
c
c  Scans in sig = sigma**2 between limits sig1 and sig2, with iscan steps,
c  for the value of l = el given in common /rhsdat/.
c  The type of scan is determined by nsig.
c  If itrsig = 1 in addition iterates for modes at zero crossings
c  of matching determinant and outputs results.
c
c  Scan is controlled by nsig:
c   nsig = 1: uniform in squared frequency (i.e., in sig)
c   nsig = 2: uniform in frequency (i.e., in sqrt(sig))
c   nsig = 3: uniform in 1/(frequency) (i.e., in 1/sqrt(sig))
c   nsig = 4: uniform in 1/(frequency) at low frequency and 
c             in frequency at high frequency.
c   nsig .ge. 10: set scan step based on asymptotic properties of frequency
c             spectrum
c
c  Original version 2/7/95
c
c  Modified 18/10/00, setting sig before call of stevft.
c
c  Modified 4/5/02, resetting fsig before iteration
c
c  Modified 14/5/02, introducing possibility of fine scan near local
c  minimum or single-point sign change in fitting determinant.
c
c  Modified 7/3/06, to include option nsig = 4.
c
c  Modified 24/7/09, allowing output of unconverged eigenfunctions
c  during simple scan, for setting core phase, flagged by setting
c  nfmode.
c
c  Modified 29/7/10, changing criterion for onset of fine scan. This
c  deserves refinement, better control.
c
c  Modified 22/8/11, including option nsig .ge. 10 to use 
c  asymptotic properties of frequency spectrum in scan.
c
c  Modified 14/3/14, introducing option nsig .ge. 20 to test for 
c  the start of the scan, based on the g-mode asymptotic integral,
c  to avoid severely underresolved modes (test hardcoded in 
c  s/r siginit_as).
c
c  Modified 18/2/16, setting test for stop of scan to kdgbcs .le. -10
c  to activate it only for problems at the outer boundary conditions.
c
c  Modified 1/3/17, introducing test for number of unconverged solutions
c  in scan. If this exceeds i_noconv_max (hard-coded below) the scan
c  stops and the routine returns with icry = -1.
c
c  Modified 8/3/18, including required shift of envelope solution
c  when outputting eigenfunction during scan, with nfmscn = 1
c
c  Modified 5/4/19, including options sig1 lt 0 and/or sig2 lt 0, to
c  limit scan to be between cut-off frequencies.
c
c  Modifications 21/5 - 5/6/19: Extensive modifications to tests for 
c  missing modes and/or singularities. All this, and zero crossing of
c  determinand, are now dealt with in s/r test_scan. This includes
c  direct flags from s/r nrkint for singularities in the normalization
c  of the discontinuity measure for mdintg = 3 (relaxation method)
c  Introduce resetting of fitting point before start of scan, to make 
c  sure that it is outside the region of evanescent treatment, 
c  if -1 < xfit_in < 0, where xfit_in is the input xfit.
c  Introduce resetting of fitting point in connection with fine scans.
c  Extend allowed number of resets of fitting point when 
c  mod(nftmax,10) = 0.
c  (Note: version of routine prior to these changes can be found in
c  directory old_190515)
c
c  Modified 9/6/19, blocking reset of fitting point when nftmax < 0
c  Also block resetting for fine scan unless nftrcs = 1 (so reset is
c  in terms of xfit, not nfit)
c
c  Modified 12/6/19, introducing a separate scan in s/r sigitr, if
c  the iteration in that routine results in a value of sig outside
c  the interval [sigi2, sigi1] identified as result of the scan.
c  Note: this is not employed on full mesh with Richardson extrapolation
c  since scan takes place on thinned mesh.
c
c  Modified 15/6/19, dropping retry of iteration if solution is 
c  unconverged.
c  Convergence issues should already be handled in s/r sigsol, etc.
c
c  ----------------------------------------------------------------------
c
c  Modifications June 2021, Substantial modifications to scanning and
c  fitting procedures:
c
c  Moving setting of next step to signew_com, handling the different
c  cases of nsig
c
c  Introduce sig_store storing the last isig_score (hard-coded to 10)
c  modes stored to file
c
c  Allow fine scan to several levels, at most ifine_max (hard-coded
c  to 3 in the routine)
c
c  When starting fine scan, make sure that lower limit exceeds the 
c  last mode written to file, to avoid duplicate modes
c
c  Introduce option |itsord| = 2, to repeat search for next mode,
c  if a mode is found that is not adjacent to the preceding mode.
c
c  --------------------------------------------------------------------
c
c  Modified 28/3/22, setting i_sing = 1 for test for fine scan (5), i.e.,
c  based on second differences. Also, block this test when 
c  isig = i_sing+1
c
c  Modified 29/3/22, introducing hard-coded flag i_exit_on_error.
c  For i_exit_on_error = 1 exit programme after return froom sigsol
c  with ierr .lt.0, ierr. neq. -5 (the old action)
c  With i_exit_on_error = 0, continue outermost scan.
c
c  ----------------------------------------------------------------------
c  
      implicit double precision (a-h, o-z)
      logical radial, fulmod
      parameter(iaa=10,isig_store=10)
      include 'adipls.c.d.incl'
      dimension x(*), y(iy,*)
      dimension ds1(4,5)
      dimension sig_store(4,isig_store)
      dimension isigst(10), isig0st(10), sigfst(10), sigpst(10),
     *  detst(10), detpst(10), iscans(10), nsigst(10), dsigst(10),
     *  i_singst(10),nevst(10),nfitst(10)
c
      common/sysord/ ii
      common/rhsdat/ el,ell,alb,els,el1,sig,anres,perfac,data(8),
     *  aa(iaa,nnmax)
      common/bcsdat/ fctsbc, fcttbc, istsbc, ibotbc
      common/cincnt/ xmnevn,xfit, fcnorm, eps, epssol, dsigmx, fsig, 
     *  dsigre,icow,iturpr,iplneq,iriche,nnwwin, moddet, itmax, irsevn, 
     *  nftmax,irsord,iekinr,itsord,imissl,imjssl,imstsl,inomd1
      common/c_nrkdiag/  signrk_p, signrk, det_norm_p, det_norm, 
     *  isig_nrk, ising_nrk, ifine_nrk,isig,iord_out
      common/c_scanctrl/ sigi1, sigi2
c
c  common controlling output
c
      common/coutpt/ nout,nprcen,iper,irotkr,nprtkr,igm1kr,npgmkr,
     *  nfmode,nfmesh,ispcpr,npout,nobs_stmx,nfmscn
c
c  common containing diagnostic flags possibly set in various parts of the 
c  code
c
      common/cdgsbr/ kdgbcs
c
c  common controlling diagnostics
c
      common/cdiagn/ idgrhs, idgrh1, iprdet, itssol, idgtss
c
c  common defining file input and output
c
      common/cdadsg/ imlds,idslog,idsgsm,idsssm,idsefn,
     *   idsrkr,idsgkr
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      data iscan_fine/25/
c
      if(istdpr.gt.0) write(istdpr,'(/'' Enter sigscn'')')
c
c  flag for exiting after error in sigsol
c
      i_exit_on_error=0
c
c  shift to soften test for correct interval
c 
      eps_tst=1.d-4
c
c  store original iscan and nsig
c
      iscan_orig=iscan
      nsig_orig=nsig
c
c  counter for stored results
c
      lsig_store=0
c
      isig_nrk=0
c
c  set ifine = -1 to disable using finer scan
c
      ifine=0
c
c  maximum levels of scan refinement
c
      ifine_max=3
      if(istdpr.gt.0) write(istdpr,'(/
     *  '' Maximum levels of mesh refinement: '',i2)') ifine_max
c
c  initialize counter for unconverged solutions and maximum number
c  of unconverged solutions
c
      i_noconv=0
      i_noconv_max=10
c
      i_retry_order=0
c
c  initialize integration parameters
c
      istsb1=istsbc
      radial = el.le.1.e-6
      fulmod = nibc.eq.2
c
c  initialize parameters for resetting nfit
c
      nftrcs=nftmax/10
      nftmx1=mod(nftmax,10)
      if(nftmax.gt.0.and.nftmx1.eq.0) nftmx1=10000
      if(istdpr.gt.0) write(istdpr,'(/
     *  '' nftmx1, nftrcs set to'',2i6)') nftmx1, nftrcs
c
c  Test for Richardson extrapolation. If so, use thinned mesh
c
      if(iriche.eq.1) then
        iasn = 2
      else
        iasn = 1
      end if
c
c  set initial (ficticious) order for comparison
c
      iord_out=-1000000
c
c  test for Cowling approximation (note that for icow = 1 scan is
c  done in Cowling approximation)
c
      if(radial.or.icow.gt.0) then
        ii = 2
        ig = 1
      else
        ii = 4
        ig = 2
      end if
c
      sig_init=sig1
      sig_fin=sig2
c
      sigmd=0.d0
c
      i_ret=0
c
c  test for limiting scan by cut-off frequencies or f-mode frequency
c
      if(sig1.le.0.or.sig2.lt.0) then
        call siglim_iso(x,nn,sig1,sig_low,sig_up,icry)
        if(sig1.le.0) sig_init=sig_low
        if(sig2.lt.0) sig_fin=sig_up
      end if
c
c  initial point of scan, possibly based on asymptotic properties
c
      sig=sig_init
      if(nsig.ge.20.and.el.ge.1.e-6) then
        call siginit_as(sig_init,nsig,x,nn,sig,icry)
      end if
c
c..      write(istdpr,*) '#D# sig_init,sig =',sig_init, sig
c
c  set initial boundary of evanescent region
c
      els=ell/sig
      call stevft(x,nn,nibc,nfit,iasn,radial,fulmod,nev1,xfit1,
     *  'sigscn')
c
c  possibly reset fitting point 
c
      call set_xfit(el,x,data,aa,iaa,nn,nibc,xfit,nev1,0.95d0,
     *  xfit1,nfit,'sigscn')
      if(istdpr.gt.0.and.xfit1.ne.xfit) write(istdpr,'(/
     *    '' xfit, x(nfit), nfit set to'',2f10.6,i6,'' in sigscn'')')
     *  xfit1, x(nfit), nfit
c
      isig=0
      isigfn=1
c
c  initialize starting point of present scan
c
      isig0=0
c
c  initialize i_sing
c
      i_sing=-1
c
c  initialize nftcnt
c
      nftcnt=0
c
c  Step for sigma*2 stepping.
c  case determined by nsig (as in function signew)
c
      iscan1=iscan-1
      if(nsig.eq.1) then
c
c  linear in squared frequency
c
        dsigs=(sig_fin-sig_init)/iscan1
      else if(nsig.eq.2) then
c
c  linear in frequency
c
        dsigs=(sqrt(sig_fin)-sqrt(sig_init))/iscan1
      else if(nsig.eq.3) then
c
c  linear in period
c
        dsigs=(1.d0/sqrt(sig_fin)-1.d0/sqrt(sig_init))/iscan1
      else if(nsig.ne.4.and.nsig.lt.10) then
c
        write(istdou,'(/
     *    '' ***** Error in s/r sigscn. nsig = '',i4,
     *    '' not allowed'')') nsig
        if(istdou.ne.istdpr.and.istdpr.gt.0) 
     *    write(istdpr,'(/'' ***** Error in s/r sigscn. nsig = '',i4,
     *    '' not allowed'')') nsig
        stop
c
      end if
c
      det=0
      detp=0
      sigp=0
      istsbp=istsbc
      istspp=istsbc
c
c  initialize flag for stopping scan
c
      istop=0
c
c  set flag for initialization in nrkint (only used for mdintg=3)
c
      initnr=1
c
      icrysc=0
      if(istdpr.gt.0) write(istdpr,'(//'' Scan in sigma**2''/)')
c
   10 continue
c
c  start of first or continuing scan
c
c --------------------------------------------------------------------
c
c  test for stopping scan (e.g., due to diagnostics from boundary condition)
c  restoring original iscan and nsig
c
      if(istop.gt.0) then
        write(istdou,'(//'' Premature stop of scan''/)')
        if(istdpr.gt.0.and.istdpr.ne.istdou) 
     *    write(istdpr,'(//'' Premature stop of scan''/)')
        iscan=iscan_orig
        nsig=nsig_orig
        if(istdpr.gt.0) write(istdpr,'(/'' Exit sigscn'')')
        return
      end if
c
c  entry for continuing scan
c
c  Note that during fine scan, nsig has temporarily been reset to 1
c
      isig=isig+1
      if(isig.ge.2) then
        sigpp=sigp
        sigp=sig
c
        sig=signew_com(1,nsig,1,0,
     *    sigp,dsigs,itrds,el,sig_init,sig_fin,iscan,isig,x,nn,icrysc)
c
c  set dsigs if needed
c
        if(nsig.gt.4) then
          dsigs=sig-sigp
        end if
      end if
c
c  Set initial values of fsig and dsigmx
c
      if(isig.eq.2.and.ifine.eq.0) then
        fsig=0.1d0*abs(sig-sigp)/sigp
        if(istdpr.gt.0) write(istdpr,'(/
     *    '' fsig reset to'',1pe13.5)') fsig
        dsigmx=0.5d0*abs(sig-sigp)/sigp
        if(istdpr.gt.0) write(istdpr,'(
     *    '' dsigmx reset to'',1pe13.5/)') dsigmx
      end if
c
      els=ell/sig
c
      detpp=detp
      detp=det
      istspp=istsbp
      istsbp=istsb1
c
c  integration
c
      ifine_nrk=ifine
      call sigint(x,y,iy,nw1,nibc,nn,mdintg,nev1,iasn,ii,ig,
     *  initnr,nfit,istsb1,isig,det,ddsol,ds1,iord,icry)
c
c  test for error in integration
c
      if(icry.lt.0.and.icry.ne.-5) then
        write(istdou,'(/'' Return (1) from sigscn with error'')')
        if(istdpr.gt.0.and.istdpr.ne.istdou) 
     *    write(istdpr,'(/'' Return (1) from sigscn with error'')')
        if(istdpr.gt.0) write(istdpr,'(/'' Exit sigscn'')')
        return
      end if
c
c  test for setting flag to stop scan
c
c  **** Note: for now, test on sig = sigma^2 to avoid stopping
c       below the g-mode critical frequency.
c       This needs to be refined.      13/4/18
c
c..      if(inomd1.ge.2.and.kdgbcs.le.-10.and.sig.ge.1) then
c..        istop=1
c..        if(istdpr.gt.0) write(istdpr,'(/'' ***** Set istop to 1'')')
c..      end if
      if(kdgbcs.le.-10.and.sig.ge.1) then
        if(inomd1.ge.2) then
          istop=1
          if(istdpr.gt.0) write(istdpr,'(/'' ***** Set istop to 1'')')
        else
          if(istdpr.gt.0) write(istdpr,'(/
     *      '' Reset isig0 to isig ='',i5)') isig
          isig0=isig
        end if
      end if
c
c  output of matching results
c
      if(istdpr.gt.0) then
        if(mdintg.ne.2) then
          write(istdpr,'(
     *    '' isig,sig,det,det/detnrm:'',i6,f17.10,1p2e11.3)')
     *     isig,sig,det,ddsol
        else
          write(istdpr,'(
     *      '' isig,sig,det,det/detnrm,order:'',i3,f16.9,1p2e11.3,i8)')
     *      isig,sig,det,ddsol,iord
        end if
      end if
c
      i_file_scan=0
      if(i_file_scan.eq.1) then
        if(ifine.eq.0) then
          write(70,'(i6,f5.1,2i5,1p2e15.7)') isig,el,nfit,nev1,sig,det
          write(75,'(i6,f5.1,2i5,1p3e15.7)') 
     *      isig,el,nfit,nev1,sig,det*det_norm,det_norm
        else if(ifine.eq.1) then
          write(71,'(i6,f5.1,2i5,1p2e15.7)') isig,el,nfit,nev1,sig,det
        else if(ifine.eq.2) then
          write(72,'(i6,f5.1,2i5,1p2e15.7)') isig,el,nfit,nev1,sig,det
        else
          write(73,'(i6,f5.1,2i5,1p2e15.7)') isig,el,nfit,nev1,sig,det
        end if
      end if
c
      i_ret=0
c
c  End for istop = 1
c
      if(istop.gt.0) go to 10
c
c  --------------------------------------------------------------------
c
c  test for iteration or finer scan
c
      if(itrsig.eq.1.and.isig.ge.2) then
c
        call test_scan(mdintg,isig0,isig,isigfn,i_sing,sigpp,
     *     sigp,sig,detpp,detp,det,sigmd,ifine,ismode,kfine,sigi1,sigi2,
     *     deti1,deti2,iscan_fine,ifine_max)
c
c  --------------------------------------------------------------------
c
c  Start block to set up for finer scan after test_scan
c
c  store parameters for current level
c
        isigst(ifine+1)=isig
        isig0st(ifine+1)=isig0
        sigfst(ifine+1)=sig
        sigpst(ifine+1)=sigp
        detst(ifine+1)=det
        detpst(ifine+1)=detp
        iscans(ifine+1)=iscan
        nsigst(ifine+1)=nsig
        dsigst(ifine+1)=dsigs
        i_singst(ifine+1)=i_sing
        nevst(ifine+1)=nev1
        nfitst(ifine+1)=nfit
c..        write(istdpr,'('' nfitst('',i1,'') set to'',i5)') ifine+1,nfit
c
c  start setting up for finer scan
c
        if(kfine.gt.0) then
c
c  set new level
c
          ifine=ifine+1
          isig0=0
          i_sing=0
          nsig=1
          iscan=iscan_fine
          dsigs=(sigi1-sigi2)/(iscan-1)
          sigp=0.d0
          isig=0
          isigfn=1
c
c  test for start before last stored model
c
          if(lsig_store.gt.0.and.sigi2.le.sig_store(3,lsig_store)) then
            sigi2_old=sigi2
            sigi2=signew_com(1,nsig,1,0,sig_store(3,lsig_store),dsigs/5,
     *        itrds,el,sig_init,sig_fin,iscan,isig,x,nn,icrysc)
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' Shift sigi2 from '',1pe15.7,'' to '',e15.7)')
     *        sigi2_old,sigi2
c
          end if
c
          sig=sigi2
c
          write(istdou,'(/'' Start fine scan'')')
          if(istdou.ne.istdpr.and.istdpr.gt.0)
     *      write(istdpr,'(/'' Fine scan level'',i3,
     *        '' from sig ='',1pe13.6,'' to'',e14.6)') 
     *        ifine,sigi2, sigi1
          istsb1=istspp
c
c  also try to reset fitting point as part of the finer scan
c  (only allowed for nftrcs = 1, i.e., with resetting in terms of x)
c
          nfit_org=nfit
          if(nftmax.gt.0.and.nftrcs.eq.1.and.
     *      (nftcnt.le.nftmx1.or.nftmx1.ge.1000)) then
            xfit1=x(nfit_org)
            xfitst=xfit1
            call rsnfit(x,nibc,nn,nev1,nfit,xfit1,xfitst,nftcnt,nftrcs,
     *        'sigscn')
          else
            if(istdpr.gt.0) write(istdpr,'(/
     *       '' Fine scan, nftcnt, nftmx1, nftrcs ='',3i5/
     *       '' no rsnfit'')') nftcnt, nftmx1, nftrcs
          end if
c
          if(istdpr.gt.0) write(istdpr,'('' '')')
c
          go to 10
c
c  End block to set up for finer scan after test_scan
c
c  Start block for finding mode during iteration
c
c  Start setting up for iteration for mode
c
        else if(ismode.eq.1) then
c
c  begin iteration
c
          if(istdpr.gt.0) write(istdpr,'(//
     *      '' Begin iteration for sig, in sigscn'')')
          sigst = sig
          fsigst=fsig
          iretry=0
c
c  entry point after possible resetting of interval for iteration
c  Reset fsig and dsigmx only based on step in outer scan
c
   20     continue
          if(ifine.eq.0) then
            fsig=0.1d0*abs(sigi2-sigi1)/sigi1
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' fsig reset to'',1pe13.5)') fsig
            dsigmx=0.5d0*abs(sigi2-sigi1)/sigi1
            if(istdpr.gt.0) write(istdpr,'(
     *        '' dsigmx reset to'',1pe13.5/)') dsigmx
          end if
          if(deti2.ne.deti1) then
            dsig=(sigi1-sigi2)*deti1/(deti2-deti1)
            sig=sigi1+dsig
          end if
          sig0=sig
          els=ell/sig
c
          iord_out_prev=iord_out
c
c  store nfit for sigsol
c
          nfit_sol=nfit
c
          call sigsol(sig,x,y,iy,nw1,nibc,nn,nnw,mdintg,nev1,nfit_sol,
     *      istsb1,iscan,itrsig,iord_out,icry,isolcv,isigcv,sigtst)
c
c  Block of diagnostics etc. after sigsol
c
c  For icry = - 5, i.e., nrkint singularity in sigitr, do fine scan
c
          if(icry.eq.-5.and.ifine.lt.ifine_max) then
c
c  start setting up for finer scan
c
            isigst(ifine+1)=isig
            isig0st(ifine+1)=isig0
            sigfst(ifine+1)=sig
            sigpst(ifine+1)=sigp
            detst(ifine+1)=det
            detpst(ifine+1)=detp
            iscans(ifine+1)=iscan
            nsigst(ifine+1)=nsig
            dsigst(ifine+1)=dsigs
            i_singst(ifine+1)=i_sing
            nevst(ifine+1)=nev1
            nfitst(ifine+1)=nfit
c..            write(istdpr,'('' nfitst('',i1,'') set to'',i5)') 
c..     *        ifine+1,nfit
c
            ifine=ifine+1
            isig0=0
            nsig=1
            iscan=iscan_fine
            dsigs=(sigi1-sigi2)/(iscan-1)
            sig=sigi2
            sigp=0.d0
            isig=0
            isig0=0
            isigfn=1
c
c..            write(istdou,'(/'' icry returned as -5 from sigsol''/
c..     *        '' Start fine scan'')')
            if(istdou.ne.istdpr)
     *        write(istdpr,'(/
     *          '' icry returned as -5 from sigsol''/
     *          '' Fine scan level'',i3,
     *          '' from sig ='',1pe13.6,'' to'',e14.6)')
     *          ifine,sigi2, sigi1
            istsb1=istspp
c
c  also try to reset fitting point as part of the finer scan
c  (only allowed for nftrcs = 1, i.e., with resetting in terms of x)
c
            nfit_org=nfit
            if(nftmax.gt.0.and.nftrcs.eq.1.and.
     *        (nftcnt.le.nftmx1.or.nftmx1.ge.1000)) then
              xfit1=x(nfit_org)
              xfitst=xfit1
              call rsnfit(x,nibc,nn,nev1,nfit,xfit1,xfitst,
     *          nftcnt,nftrcs,'sigscn')
            else
              if(istdpr.gt.0) 
     *          write(istdpr,'(/'' Fine scan, no rsnfit'')')
            end if
c
            go to 10
c
          else if(icry.lt.0) then
            if(istdpr.gt.0) write(istdpr,
     *        '(/'' Return (2) from sigscn with error'',i4)') icry
            if(i_exit_on_error.eq.1) then
              if(istdpr.gt.0) write(istdpr,'('' Exit sigscn'')')
              return
            else
              if(istdpr.gt.0) write(istdpr,
     *          '('' Continue outer scan'')')
              ifine=0
              iscan=iscans(ifine+1)
              isig=isigst(ifine+1)
              isig0=isig0st(ifine+1)
              isigfn=isig
              sig=sigfst(ifine+1)
              sigp=sigpst(ifine+1)
              det=detst(ifine+1)
              detp=detpst(ifine+1)
              dsigs=dsigst(ifine+1)
              nsig=nsigst(ifine+1)
              i_sing=i_singst(ifine+1)
              nev1=nevst(ifine+1)
              nfit=nfitst(ifine+1)
              xfit=x(nfit)
              go to 10
            end if
          end if
c
c  End block of diagnostics etc. after sigsol
c
          sigmd=sig
c
c  Drop retry with failed convergence here.
c  Already handled in s/r sigsol (15/6/19)
c
c  Test that the solution (on thinned mesh, with RI)
c  is in the correct interval.
c  Otherwise, try again.
c
          sig1_tst=(1.d0-eps_tst)*min(sigi1,sigi2)
          sig2_tst=(1.d0+eps_tst)*max(sigi1,sigi2)
          if((sigtst.lt.sig1_tst.or.sigtst.gt.sig2_tst).and.
     *      isolcv.gt.0.and.iretry.le.3) then
c
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' ***** Warning. Converged thinned sig ='',1pe13.5/
     *        ''       not in correct interval: ('',2e13.5,'')''//
     *        ''       Try new iteration.''/)') sigtst,sig1_tst,sig2_tst
c
            iretry=iretry+1
            if(istdpr.gt.0) write(istdpr,'(/'' iretry = '',i3)') iretry
            sig=sig0
            ifine_nrk=ifine
            call sigint(x,y,iy,nw1,nibc,nn,mdintg,nev1,iasn,ii,ig,
     *        initnr,nfit,istsb1,isig,det0,ddsol,ds1,iord,icry)
c
c  test for interval with change of sign
c
            if(det0*deti1.lt.0) then
              deti2=det0
              sigi2=sig0
            else
              deti1=det0
              sigi1=sig0
            end if
c
            go to 20
c
          end if
c
          fsig=fsigst
c
          if(isolcv.le.0) then
            write(istdou,
     *        '(/'' ***** Output for unconverged solution.'',
     *        '' i_noconv = '',i3)') i_noconv
            if(istdpr.gt.0.and.istdou.ne.istdpr) write(istdpr,
     *        '(/'' ***** Output for unconverged solution.'',
     *        '' i_noconv = '',i3)') i_noconv
            i_noconv=i_noconv+1
          end if
c
c  call output routine
c
          call sigout(sig,x,y,iy,nw1,nibc,nn,nnw,mdintg,nev1,nfit,
     *      istsb1,iord_out,isolcv,isigcv,icry)
c
c  test for going back to previously converged mode and restart 
c  scan with different fitting point
c
          if(icry.eq.-8) then
c
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' Retry scan for missing mode(s) going back to isig ='',
     *        i6)') isigst_conv
            i_retry_order=1
            isig=isigst_conv
            isig0=isig
            sig=sigfst_conv
            dsig=dsigst_conv
            nsig=nsigst_conv
            i_sing=i_singst_conv
            ifine=0
c
            nfit_org_conv=nfit
            xfitst=xfit1
            call rsnfit(x,nibc,nn,nev1,nfit,xfit1,xfitst,
     *        nftcnt,nftrcs,'sigscn')
c
            go to 10
          end if
c
          if(i_noconv.ge.i_noconv_max) then
            write(istder,'(/'' ***** Stop scan after'',i3,
     *        '' unconverged solutions'')') i_noconv
            write(istdou,'(/'' ***** Stop scan after'',i3,
     *        '' unconverged solutions'')') i_noconv
            if(istdpr.gt.0.and.istdou.ne.istdpr)
     *        write(istdpr,'(/'' ***** Stop scan after'',i3,
     *        '' unconverged solutions'')') i_noconv
            icry=-1
            if(istdpr.gt.0) write(istdpr,'(/'' Exit sigscn'')')
            return
          end if
c
c  store values for possible retrying scan with missing order
c  Note that this goes back to the basic scan
c
          isigst_conv=isigst(1)
c..          sigfst_conv=sigfst(1)
          dsigst_conv=dsigst(1)
          sigfst_conv=sigtst+0.1*dsigst_conv
c..          write(istdpr,*) '#D#', sigfst_conv,sigtst,dsigst_conv
c..          if(iord_out.eq.1) stop
          nsigst_conv=nsig_orig
          i_singst_conv=i_sing
          i_retry_ord=0
          if(sigfst_conv.le.sigtst) then
            sigfst_old=sigfst_conv
            sigfst_conv=signew_com(1,nsigst_conv,1,0,sigfst_conv,
     *        dsigst_conv,
     *        itrds,el,sig_init,sig_fin,iscan,isig,x,nn,icrysc)
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' Shift sigfst_conv from '',1pe15.7,'' to '',e15.7)')
     *        sigfst_old,sigfst_conv
          end if
c
c  test for resuming after retrying scan for missing order
c
          if(i_retry_ord.eq.1.and.iord_out_prev.gt.-1000000.and.
     *      ord_out.eq.iord_out_prev+1) then
            i_retry_ord=0
            isig0=isig
            nfit=nfit_org_conv
c
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' Continue scan after retry for missing mode'')')
          end if
c
c  update sig_store
c
          if(icry.ne.-7) call store_sig(sig_store,isig_store,lsig_store,
     *      el,iord_out,sigtst,sig)
c
c  continue scan after iteration
c
          sig = sigst
c
          istsb1=istsbc
          initnr=1
          i_ret = 1
          isig_nrk=0
c
c  possibly reset initial boundary of evanescent region
c
          els=ell/sig
          nfit_old=nfit
          call stevft(x,nn,nibc,nfit,iasn,radial,fulmod,nev1,xfit1,
     *      'sigscn')
c
          if(nfit.ne.nfit_old) then
            isig0st(ifine+1)=isig0
            isig0=isig
            nfit_org=nfit
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' nfit reset from '',i6,'' to '',i6/
     *        '' Reset starting point for scan test to isig0 = '',i5)')
     *        nfit_old,nfit,isig0
          end if
c
          if(istdpr.gt.0) then
            write(istdpr,'(//1x,30(1h*)/
     *        '' Continue scan in sig, ifine level'',i3,
     *        '' isig0 ='',i4)') ifine, isig0
            write(istdpr,'('' nfit, nev ='',2i6/)') nfit,nev1
          end if
c
        end if
c
c  End block for finding mode during iteration
c
c  end of section for iteration for solution
c  Test for output of eigenfunction during scan with no iteration
c
      else if(nfmscn.gt.0) then
c
c  call output routine to output eigenfunction
c  (might need some cleaning up later)
c  When using shooting method, shift envelope solution
c  (corrected 8/3/18)
c
        if(mdintg.ne.3) then
          if(nfit.le.0) then
            nshft=1
          else
            nshft=nfit+1
          end if
          do n=nshft,nn
            do i=1,2
              y(i,n)=y(i,n+1)
            end do
          end do
        end if
c
        isigcv=1
        isolcv=1
        call sigout(sig,x,y,iy,nw1,nibc,nn,nnw,mdintg,nev1,nfit,
     *    istsb1,iord,isolcv,isigcv,icry)
        initnr=1
      end if
c
c  continue scan
c
      if((nsig.lt.10.and.isig.lt.iscan).or.
     *    nsig.ge.10.and.icrysc.eq.0) then
        go to 10
c
c  test for continuing after fine scan
c
      else if(ifine.ge.1) then
        if(istdpr.gt.0) then
          if(ifine.gt.1) then
            write(istdpr,'(/'' End fine scan level'',i3/
     *        '' Continue fine scan at level'',i3,
     *        '' with nfit, nev ='',2i6)') ifine,ifine-1, nfit, nev1
          else
            write(istdpr,'(/'' End fine scan level'',i3/
     *        '' Continue normal scan with nfit, nev ='',2i6)')
     *        ifine, nfit, nev1
          end if
        end if
        ifine=ifine-1
        iscan=iscans(ifine+1)
        isig=isigst(ifine+1)
        isig0=isig0st(ifine+1)
        isigfn=isig
        sig=sigfst(ifine+1)
        sigp=sigpst(ifine+1)
        det=detst(ifine+1)
        detp=detpst(ifine+1)
        dsigs=dsigst(ifine+1)
        nsig=nsigst(ifine+1)
        i_sing=i_singst(ifine+1)
        nev1=nevst(ifine+1)
        nfit=nfitst(ifine+1)
        xfit=x(nfit)
c
c..        nfit=nfit_org
        i_ret = 2
        isig_nrk=0
        go to 10
c
      end if
c
      if(istdpr.gt.0) write(istdpr,'(/
     *  '' Exit sigscn at end of routine'')')
      return
      end
      subroutine siginit_as(sig1,nsig,x,nn,sigst,icry)
c
c  Sets first step in scan based on asymptotic properties of solution.
c  Minimum no of points per node has been hardcoded to 10
c
c  Original version 14/3/14
c
      implicit double precision (a-h, o-z)
      include 'adipls.c.d.incl'
      parameter(iaa=10,iw=8)
      dimension x(*)
c
      common/rhsdat/ el,ell,alb,els,el1,sig,anres,perfac,data(8),
     *  aa(iaa,nnmax)
      common/work/ xi(nnmax)
      common/worksp/ wrk(iw,nnmax)
c
c  common containing diagnostic flags possibly set in various parts of the 
c  code
c
      common/cdgsbr/ kdgbcs
c
c  common controlling diagnostics
c
      common/cdiagn/ idgrhs, idgrh1, iprdet, itssol, idgtss
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      pi=4.d0*atan(1.d0)
      kmin=10
c
c  set integrand, excluding centre
c
      n1=0
      do n=2,nn
        bvsq=aa(1,n)*abs(aa(4,n))
        wrk(1,n)=sqrt(bvsq)/x(n)
      end do
c
      wrk(1,1)=wrk(1,2)
c
c  trapezoidal integration
c
      wrk(2,1)=0.d0
      do n=2,nn
        wrk(2,n)=wrk(2,n-1)+0.5d0*(x(n)-x(n-1))*(wrk(1,n-1)+wrk(1,n))
      end do
c
      sigmin=sqrt(ell)*kmin*wrk(2,nn)/(nn*pi)
      sigmin_sq=sigmin*sigmin
c..      write(istdpr,*) 'l, wrk(2,nn), sigmin_sq',el, wrk(2,nn), sigmin_sq
c
      if(sigmin_sq.le.sig1) then
        sigst=sig1
        icry=0
      else
        sigst=sigmin_sq
        icry=1
        write(istdou,'(//
     *    '' In siginit_as reset starting squared frequency'',
     *    '' to avoid excessive mode orders''/
     *    '' Original value: '',1pe13.5,'' new value: '',e13.5)')
     *    sig1, sigmin_sq
        if(istdpr.gt.0.and.istdpr.ne.istdou) write(istdpr,'(//
     *    '' In siginit_as reset starting squared frequency'',
     *    '' to avoid excessive mode orders''/
     *    '' Original value: '',1pe13.5,'' new value: '',e13.5)')
     *    sig1, sigmin_sq
      end if
      return
      end
      subroutine siglim_iso(x,nn,sig1,sig_low,sig_up,icry)
c
c  sets limits of interval [sig_low, sig_up] in squared dimensionless 
c  frequency between the lower (gravitational) and upper (acoustic)
c  cut-off frequencies.
c
c  Original version: 5/4/19
c
c  Modified 12/2/21 resetting sig_low from sig_up to avoid round-off errors
c  Modified 1/3/22, introducing option sig1 = 0 to start at nominal 
c  f-mode value sig = l, when bigger than sig_lim (hardcoded to 4) or
c  sig_lim otherwise
c
      implicit double precision (a-h, o-z)
      include 'adipls.c.d.incl'
      parameter(iaa=10,iw=8)
      dimension x(*)
c
      common/rhsdat/ el,ell,alb,els,el1,sig,anres,perfac,data(8),
     *  aa(iaa,nnmax)
c
c  common containing diagnostic flags possibly set in various parts of the 
c  code
c
      common/cdgsbr/ kdgbcs
c
c  common controlling diagnostics
c
      common/cdiagn/ idgrhs, idgrh1, iprdet, itssol, idgtss
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      idiag=1
c
      sig_lim=3.d0
c
      ai=aa(2,nn)*(aa(3,nn)-1)
      dd=(ell+aa(2,nn)*ai+0.25d0*(ai+4.d0-aa(2,nn))**2)/aa(2,nn)
      dds=sqrt(0.25d0*dd**2-ai*ell/aa(2,nn))
      sig_up=(0.5d0*dd+dds)/x(nn)**3
      if(el.lt.1.d-6) then
        sig_low=sig_lim
      else
c..        sig_low=(0.5d0*dd-dds)/x(nn)**3
c
c  To avoid round-off errors, use sig_max.
c  Also scale to correct for remaining errors
c
        sig_low=(1+1.d-6)*ai*ell/(x(nn)**6*aa(2,nn)*sig_up)
      end if
c
c  For sig1 = 0, replace sig_low by f-mode estimate
c
      if(sig1.eq.0) sig_low=max(sig_lim,sqrt(ell))
c  
      if(istdpr.gt.0.and.idiag.gt.0) then
        write(istdpr,'(/
     *    '' Squared dimensionless cut-off frequencies:''/
     *    '' sigma_low^2 = '',1pe13.5/
     *    '' sigma_up^2  = '',e13.5)') sig_low, sig_up
        if(sig1.eq.0.and.sig_low.gt.sig_lim) write(istdpr,'(
     *    '' sigma_low^2 set from f-mode estimate'')')
      end if
c
      icry=0
      return
      end
      subroutine test_scan(mdintg,isig0,isig,isigfn,i_sing,sigpp,
     *  sigp,sig,detpp,detp,det,sigmd,ifine,ismode,kfine,sigi1,sigi2,
     *  deti1,deti2,iscan_fine,ifine_max)
c
c  Analyse results of scan, including testing for start of iteration
c  or start of fine scan
c
c  isig0 marks the starting point of the present sequence of scans, 
c  originally 0, but possibly reset after reset of nfit
c
c  Original version: 25/5/19
c
c  Modified 7/6/21, introducing isig0
c
c  Modified 28/2/22, extending nscan_store, correcting error
c  in setting second difference in scan_store(.,3)
c
c  Modified 21/3/22, adding third dimension to scan_store to separate
c  different levels
c
      implicit double precision(a-h, o-z)
      parameter(nscan_max=10000)
      dimension scan_store(10,nscan_max,5)
      common/c_nrkdiag/  signrk_p, signrk, det_norm_p, det_norm, 
     *  isig_nrk, ising_nrk, ifine_nrk,isig_diag,iord_out
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
c..      data iscan_fine1, iscan_fine2 /25, 50/
      data iscan_fine1, iscan_fine2 /12, 25/
      data ismode_p /0/
      save
c
      idiag = 1
      if(istdpr.le.0) idiag=0
c
c..      write(istdpr,*) '#D# In test_scan:',detpp,detp,det,isig,ising
c
      ismode_p=ismode
      ismode=0
      kfine=0
c
c  Change to apply to any level of fine scan
c  21/3/22
c  but only zero initial values if during fine scan
c
c..      if(ifine.eq.0) then
      if(ifine.ge.0) then
        if(isig.eq.isig0+1) then
          if(ifine.eq.0) then
            nzero=nscan_max
          else
            nzero=iscan_fine2
          end if
          do i=1,10
            do j=1,nzero
              scan_store(i,j,ifine+1)=0.d0
            end do
          end do
        end if
        scan_store(1,isig,ifine+1)=sig
        scan_store(2,isig,ifine+1)=det
      end if
      if(isig.gt.isig0+3.and.idiag.ge.2) 
     *  write(istdpr,'(/'' scan_store:''/(i4,1p3e13.5))')
     *  (j,(scan_store(i,j,ifine+1),i=1,3),j=isig-3,isig)
c
      if(isig.eq.isig0+1) return
c
c  If not using relaxation, test for possible mode for isig = 2
c     
      if(isig.eq.isig0+2) then
        if(mdintg.ne.3.and.det*detp.lt.0) then
          sigi1=sig
          sigi2=sigp
          deti1=det
          deti2=detp
          ismode=1
          if(idiag.gt.0) write(istdpr,'(/
     *      '' Likely mode (0) between '',1pe15.7,'' and'',e15.7/
     *      '' isig, i_sing, detpp, detp, det ='',2i5,1p3e12.4/
     *      '' sigpp, sigp, sig ='',4e13.5)')
     *      sigi2,sigi1,isig,i_sing,detpp,detp,det,sigpp,sigp,sig
        end if
        return
      end if
c
c  corrected 28/2/22, from out-commented version
c
c..      scan_store(3,isig)=scan_store(2,isig-2)-2.d0*scan_store(2,isig)+
c..     *  scan_store(2,isig)
      if(ifine.ge.0.and.isig.gt.isig0+2) then
        scan_store(3,isig,ifine+1)=
     *    scan_store(2,isig-2,ifine+1)-
     *    2.d0*scan_store(2,isig-1,ifine+1)+scan_store(2,isig,ifine+1)
      end if
c
c  test for fine scan at singularity in nrkint
c
      ignore_ising_nrk=1
      if(mdintg.eq.3.and.ising_nrk.eq.1) then
c
c  for now ignore singularity
c
        if(ignore_ising_nrk.eq.1) then
          if(istdpr.gt.0) write(istdpr,'(
     *      '' Note 23/2/22: for now ignore in sigscn -> test_scan'')')
        else
          i_sing=isig
          if(idiag.gt.0) then
            write(istdpr,'(/
     *        '' Singularity flagged in s/r nrkint.'',
     *        ''  i_sing set to '',i4)') i_sing
            write(istdpr,'(/'' Fine scan (6)'')')
          end if
          kfine=6
          sigi1=sig
          sigi2=sigp
          iscan_fine=iscan_fine1
          scan_store(5,isig,ifine+1)=kfine
          scan_store(6,isig,ifine+1)=sigi2
          scan_store(7,isig,ifine+1)=sigi1
          return
        end if
      end if
c
c  test for zero current determinant
c
      if(det.eq.0) then
        sigi1=sig
        sigi2=sig
        deti1=det
        deti2=detp
        ismode=1
        scan_store(4,isig,ifine+1)=ismode
        scan_store(6,isig,ifine+1)=sigi2
        scan_store(7,isig,ifine+1)=sigi1
        if(idiag.gt.0) write(istdpr,'(/
     *    '' Likely mode (1) between '',1pe15.7,'' and'',e15.7/
     *    '' isig, i_sing, detpp, detp, det ='',2i5,1p3e13.5/
     *    '' sigpp, sigp, sig ='',4e13.5)')
     *      sigi2,sigi1,isig,i_sing,detpp,detp,det,sigpp,sigp,sig
        return
      end if
c
c  test for possible mode between sig and sigp
c
      if(detp*det.lt.0.and.(ismode_p.eq.1.or.detp*detpp.ge.0)) then
c
c  test for singularity
c
        if(abs(detp).gt.abs(detpp)) then
          i_sing=isig
          if(idiag.gt.0) then
            write(istdpr,'(/
     *        '' Possible singularity (1); detpp, detp, det ='',
     *        1p3e13.5)') detpp, detp, det
            write(istdpr,'('' i_sing set to '',i4)') i_sing
          end if
          return
        else if(isig.gt.isig0) then
c
c  likely mode between sig and sigp
c
          sigi1=sig
          sigi2=sigp
          deti1=det
          deti2=detp
          ismode=1
          scan_store(4,isig,ifine+1)=2
          scan_store(6,isig,ifine+1)=sigi2
          scan_store(7,isig,ifine+1)=sigi1
          if(idiag.gt.0) write(istdpr,'(/
     *      '' Likely mode (2) between '',1pe15.7,'' and'',e15.7/
     *      '' isig, i_sing, detpp, detp, det ='',2i5,1p3e13.5/
     *      '' sigpp, sigp, sig ='',4e13.5)')
     *      sigi2,sigi1,isig,i_sing,detpp,detp,det,sigpp,sigp,sig
          return
        end if
      end if
c
      if(det*detp.ge.0.and.detp*detpp.lt.0) then
c
c  test for singularity between sigp and sigpp
c
        if(abs(detp).gt.abs(det)) then
          i_sing=isig-1
          if(idiag.gt.0) then
            write(istdpr,'(/
     *        '' Possible singularity (2); detpp, detp, det ='',
     *        1p3e13.5)') detpp, detp, det
            write(istdpr,'('' i_sing set to '',i4)') i_sing
          end if
          return
c
c  test for fine scan or iteration after possible false singularity
c
        else if((isig.eq.i_sing+1.or.isig.eq.3).and.
     *    abs(detp).lt.abs(det)) then
          if(ifine.le.ifine_max) then
            kfine=1
            if(idiag.gt.0) write(istdpr,'(/'' Fine scan (1)''/
     *        '' isig, i_sing, detpp, detp, det ='',i5,i3,1p3e13.5/
     *        '' sigpp, sigp, sig, sigmd ='',4e13.5)')
     *        isig, i_sing, detpp, detp, det, sigpp, sigp, sig, sigmd
            sigi1=sigp
            sigi2=sigpp
            iscan_fine=iscan_fine1
            scan_store(5,isig,ifine+1)=kfine
            scan_store(6,isig,ifine+1)=sigi2
            scan_store(7,isig,ifine+1)=sigi1
            return
c
c  try iteration between sigpp and sigp
c
          else
            sigi1=sigp
            sigi2=sigpp
            deti1=detp
            deti2=detpp
            ismode=1
            if(idiag.gt.0) write(istdpr,'(/
     *        '' Likely mode (3) between '',1pe15.7,'' and'',e15.7/
     *        '' isig, i_sing, detpp, detp, det ='',i5,i3,1p3e13.5/
     *        '' sigpp, sigp, sig ='',4e13.5)')
     *        sigi2,sigi1,isig,i_sing,detpp,detp,det,sigpp,sigp,sig
            return
          end if
        end if
      end if
c
      if(ifine.ge.ifine_max) return
c..     *  (i_sing.ne.0.and.isig.le.i_sing+2)) return
c
c  tests for fine scan
c
      if(detpp*det.gt.0) then
        if(detp*det.lt.0) then
          kfine=2
          if(idiag.gt.0) write(istdpr,'(/'' Fine scan (2)''/
     *      '' isig, detpp, detp, det ='',i5,1p3e13.5/
     *      '' sigpp, sigp, sig, sigmd ='',4e13.5)')
     *      isig, detpp, detp, det, sigpp, sigp, sig, sigmd
          if(sigmd.lt.sigpp.and.(ifine.gt.0.or.i_sing.lt.isig-1)) then
            sigi1=sig
            sigi2=sigpp
            iscan_fine=iscan_fine2
          else
            sigi1=sig
            sigi2=sigp
            iscan_fine=iscan_fine1
          end if
          scan_store(5,isig,ifine+1)=kfine
          scan_store(6,isig,ifine+1)=sigi2
          scan_store(7,isig,ifine+1)=sigi1
          return
        end if
c
        if(detpp+det.ne.0) then
          det_rat=2*detp/(detpp+det)
        else
          det_rat=1.d10
        end if
        if(det_rat.le.0.7.or.det_rat.ge.1.5) then
          kfine=3
          if(idiag.gt.0) write(istdpr,'(/'' Fine scan (3)''/
     *      '' isig, detpp, detp, det, det_rat ='',i5,1p4e13.5)')
     *      isig, detpp, detp, det, det_rat
c..          write(istdpr,*) '#D# isig, isigfn =',isig,isigfn
          if(isig.eq.isigfn+1.and.ifine.eq.0) then
            sigi2=sigp
            iscan_fine=iscan_fine1
          else
            sigi2=sigpp
            iscan_fine=iscan_fine2
          end if
          sigi1=sig
          scan_store(5,isig,ifine+1)=kfine
          scan_store(6,isig,ifine+1)=sigi2
          scan_store(7,isig,ifine+1)=sigi1
          return
        end if
c
c  21/3/22: Tighten this criterion
c
c..        if(abs(detp).lt.min(abs(detpp),abs(det))) then
        if(abs(detp).lt.0.95*min(abs(detpp),abs(det))) then
          kfine=4
          if(idiag.gt.0) write(istdpr,'(/'' Fine scan (4)''/
     *      '' isig, detpp, detp, det ='',i5,1p3e13.5)')
     *      isig, detpp, detp, det
          sigi2=sigpp
          sigi1=sig
          iscan_fine=iscan_fine2
          scan_store(5,isig,ifine+1)=kfine
          scan_store(6,isig,ifine+1)=sigi2
          scan_store(7,isig,ifine+1)=sigi1
          return
        endif
      endif
c
c  test on second difference of det (switched off for now!)
c  Switch on 21/3/22
c
c  Try drastically different test.
c
c..      if(isig.gt.isig0+5.and.ifine.eq.0.and.
c..     *  max(abs(scan_store(3,isig,ifine+1)),
c..     *  abs(scan_store(3,isig-1,ifine+1)),
c..     *  abs(scan_store(3,isig-2,ifine+1))).ge.
c..     *  1.5*abs(scan_store(3,isig-3,ifine+1))) then
      dd2_1=scan_store(3,isig,ifine+1)-scan_store(3,isig-1,ifine+1)
      dd2_2=scan_store(3,isig-1,ifine+1)-scan_store(3,isig-2,ifine+1)
      dd2_3=scan_store(3,isig-2,ifine+1)-scan_store(3,isig-3,ifine+1)
      if(isig.gt.isig0+5.and.ifine.ge.0.and.dd2_1*dd2_2.lt.0.and.
     *  max(abs(dd2_1),abs(dd2_2)).gt.3*abs(dd2_3).and.isig.gt.i_sing+1)
     *  then
        kfine=5
        i_sing=isig
        if(idiag.gt.0) write(istdpr,'(/'' Fine scan (5)''/
     *    '' isig, detpp, detp, det ='',i5,1p3e13.5/
     *    '' sigpp, sigp, sig ='',3e15.7/
     *    '' d2det ='',4e13.5/'' i_sing set to isig'')')
     *    isig, detpp, detp, det, sigpp, sigp, sig,
     *    (scan_store(3,i,ifine+1),i=isig-3,isig)
c..        write(istdpr,'(a,1p3e13.5)') '#D# dd2_3 etc.',dd2_3,dd2_2,dd2_1
        sigi2=sigpp
        sigi1=sig
        scan_store(5,isig,ifine+1)=kfine
        scan_store(6,isig,ifine+1)=sigi2
        scan_store(7,isig,ifine+1)=sigi1
        iscan_fine=iscan_fine2
        return
      endif
      return
      end
      subroutine store_sig(sig_store,isig_store,lsig_store,el,iord,
     *  sigtst,sig)
c
c  Updates sig_store, containing the up to isig_store last results
c  store to file.
c  sig_store(1,2): l, order
c  sig_store(3): Possible result on thinned mesh
c  sig_store(4): Final sig
c
c  Original version: 21/6/21
c
      implicit double precision(a-h, o-z)
      dimension sig_store(4,isig_store)
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      idiag=1
c
c   test for shifting sig_store
c
      if(lsig_store.eq.isig_store) then
        do i=2,lsig_store
          do j=1,4
            sig_store(j,i-1)=sig_store(j,i)      
          end do
        end do
c
      else
c
        lsig_store=lsig_store+1
      end if
c
      sig_store(1,lsig_store)=el
      sig_store(2,lsig_store)=iord
      sig_store(3,lsig_store)=sigtst
      sig_store(4,lsig_store)=sig
c
c  test for diagnostic output
c
      if(idiag.gt.0.and.istdpr.gt.0) write(istdpr,'(/
     *  '' sig_store:''/
     *  ''      l      n     sig(thin)         sig''/
     *  (0p2f8.1,1p2e15.7))')
     *  ((sig_store(j,i),j=1,4),i=1,lsig_store)
c
      return
      end
