      subroutine takata(x,y,nn,iasn,data,aa,iy,iaa,iord,sig)
!
!  this subroutine intends(!) to define the Takata's variables given
!  in Takata,M. 2006a, ESASP, 624, 26 (section3) and to identify the 
!  modes according to the scheme therein.
!
!  Dated Feb2008
!
!  Modified 5/5/10 setting nnmax with include file and using consistent
!  value of G
!
!  Modified 29/7/10, accounting for using every second mesh point with
!  Richardson extrapolation.
!  Also fixing dividing by zero gravity at core, in setting T2.
!
!  Modified 15/9/10, fixing dividing by zero gravity at centre, in
!  setting yy(2,.)
!
!  Modified 11/10/10, correcting the use of aa(i,.), replacing 
!  aa(i,.) by aa(i-1,.) and aa(1,.) by x(.).
!
!  Modified 24/1/13, replacing eigenfunctions by eigenfunctions proposed
!  by Tataka (2006b; PASJ 58, 893), Eqs (69) and (70), which appear to give 
!  more stable classification.
!
!  Modified 16/10/13, forcing g-mode node when r/R .le. 1.d-3. May need
!  further checking.
!
!  Modified 15/2/19, switching out forcing of g-mode node in core,
!  setting a limit xg_lim = 0. Check whether this gives problems
!  elsewhere
!
!  Modified 22/8/19, including the modified characteristic frequencies
!  in the test for the nature of the node, in the updated procedure.
! 
!  Modified 4/9/19, multiplying eigenfuctions yy based on Takata(2006a),
!  by 3, relative to the definition in the paper. This makes them
!  consistent with CY^a in 'Notes on adiabatic oscillation programme'.
!  Also, divide eigenfunctions by x, for consistency with CY in Notes.
!  Finally, normalize eigenfunctions output to unit 99 by y_1 on 
!  surface, for consistency with eigenfunctions output by the code.
!
!  Modified 6/9/19, major modification. Analyse Cy and CY^a in parallel,
!  with preference given to CY^a.
!  If number of zeros in CY1 and CY1^a are different, use CY^a
!  Otherwise, at zeros with different classification for CY and CY^a,
!  set phase sign from location of frequency relative to Takata-modified
!  characteristic frequencies.
!
      IMPLICIT NONE
      integer iaa,iord,iordg,iordp,iy,i,j,k,k1,n,nn,iasn,nnmax,ns,
     *        istdin,istdou,istdpr,istder,icount,
     *        klim,nordp,nordg,icount_p,i_charfreq,
     *        nzer,nzer1,nobs_stmax,istsbc,ibotbc
c..      parameter (nnmax=10000)
      include 'adipls.c.d.incl'
      real*8 aa(iaa,nn),data(*),dfyy,dfyy1,G,JJ(nnmax),phx1,phx2,
     *       phy1,phy2,phy2m,phx11,phx12,phy11,phy12,phy12m,
     *       pi,rho(nnmax),sig,T2(nnmax),x(nn),
     *       y(iy,nn),yy(2,nnmax),gr(nnmax),dphipdr(nnmax), cgrav,
     *       yy1(2,nnmax),xg_lim,Jsq(nnmax),sltil(nnmax),bvtil(nnmax),
     *       y1s,fctsbc, fcttbc
      integer kz(nnmax),kz1(nnmax),kzerp(nnmax),kzerg(nnmax)
      common/ccgrav/ cgrav
      common/bcsdat/ fctsbc, fcttbc, istsbc, ibotbc
      common/corder/ nordp, nordg
      common /cstdio/ istdin, istdou, istdpr, istder
      data icount /0/
      save
c
c  limit in x to force g-model node
c
      xg_lim=0.d0

      icount_p=0
      if(istdpr.le.0) icount_p=0
      icount=icount+1
!
c..      if(icount.eq.1) open(87,file='ttt.eigen.out')
!
      pi=3.141593d0
      G=cgrav
!
      do n=1,nn
        if(n.eq.1.or.mod(n-2,iasn).eq.0) then
          rho(n)=data(1)*aa(1,n)*aa(5,n)/(4*pi*(data(2))**3)
          JJ(n)=1-aa(5,n)/3
          Jsq(n)=JJ(n)*JJ(n)
          if(x(n).gt.0) then
            sltil(n)=2*aa(1,n)*Jsq(n)/aa(2,n)
            bvtil(n)=aa(1,n)*aa(4,n)/Jsq(n)
          else
            sltil(n)=0
            bvtil(n)=0
          end if
          gr(n)=G*aa(1,n)*x(n)*data(1)/data(2)**2
!       gr: gravitational acceleration
          if(x(n).gt.0) then
            dphipdr(n)=-(gr(n)/x(n))*(y(4,n)-y(3,n))-(y(3,n)*G*data(2)*
     *        4*pi*rho(n))
!       dphipdr is d(phi')/dr 
            T2(n)=-y(3,n)/(3*x(n))-dphipdr(n)/(3*gr(n))
!       T2 is the 2nd term in both of the new variables
            yy(1,n)=(JJ(n)*y(1,n)/x(n))+T2(n)
            yy(2,n)=JJ(n)*y(2,n)*G*data(1)*sig/(2*gr(n)*data(2)**2)+
     *        JJ(n)*y(3,n)/x(n)+T2(n)   
c
c  For consistency with Notes on ..., multiply by 3.
c  Then these correspond to CY^a in Notes.
c
            yy(1,n)=3.d0*yy(1,n)
            yy(2,n)=3.d0*yy(2,n)
c
c  set alternative functions (as presented in Aerts et al.)
c  Division by x(n) added 4/9/19. Then these correspond to
c  CY, as defined in Notes.
c
            yy1(1,n)=((3.d0-aa(5,n))*y(1,n)-
     *        (2.d0-aa(5,n))*y(3,n)+y(4,n))/x(n)
            yy1(2,n)=(aa(2,n)*aa(3,n)*(-aa(1,n)*y(1,n)+sig*y(2,n)/2.d0
     *       +aa(1,n)*y(3,n))/aa(1,n))/x(n)
c
          end if
        end if
      end do
c
c..      write(87,'(i3,1p5e13.5)') (icount,x(n),yy(1,n),yy(2,n),
c..     *  yy1(1,n),yy1(2,n),n=1,nn)
      if(x(1).eq.0) then
        dphipdr(1)=dphipdr(2)
        T2(1)=T2(2)
        yy(1,1)=yy(1,2)
        yy(2,1)=yy(2,2)
      end if
!
!
!  limit for start of search for zeros
!
      klim=2+iasn
      nzer=0
      nzer1=0
c
c  set zeros of CY1^a and CY1
c
      do k=1,nn-1
        if((k.eq.1.or.mod(k-2,iasn).eq.0).and.k.ge.klim) then
          phx1=yy(1,k)
          phx2=yy(1,k+iasn)
          phx11=yy1(1,k)
          phx12=yy1(1,k+iasn)
          if(phx1*phx2.le.0) then
            nzer=nzer+1
            kz(nzer)=k
          end if
          if(phx11*phx12.le.0) then
            nzer1=nzer1+1
            kz1(nzer1)=k
          end if
        end if
      end do
c
      i_charfreq=0
c
c  test for same number of zeros
c
      if(nzer.ne.nzer1) then
c
        if(istdpr.gt.0) write(istdpr,'(/
     *    '' ***** Warning: different no of zeros in CY1^a and CY1''/
     *    ''       Set order from CY^a''/)')
c
c  base order solely on CY^a
c
        iordp=0
        iordg=0
        do j=1,nzer
          k=kz(j)
          phx1=yy(1,k)
          phy1=yy(2,k)
          phx2=yy(1,k+iasn)
          phy2=yy(2,k+iasn)
!
          phy2m=(phx1*phy2-phx2*phy1)/(phx1-phx2)
          dfyy=phy2m*(phx2-phx1)
          if (dfyy.LT.0) then
            iordp=iordp+1
            if(icount.le.icount_p) 
     *        write(istdpr,'(''p:'',i6,1p4e13.5,i6)') 
     *        k, x(k), phx1, phx2, phy2m, iordp
          else if (phx1*phx2.LE.0.and.dfyy.gt.0) then
            iordg=iordg+1
            if(icount.le.icount_p) 
     *        write(istdpr,'(''g:'',i6,1p4e13.5,i6)') 
     *        k, x(k), phx1, phx2, phy2m, iordg
          end if
        end do
c
        if (iordp.GE.iordg) then 
          iord=iordp-iordg+1
        else 
          iord=iordp-iordg
        end if
c
        if(istdpr.gt.0) write(istdpr,
     *    '(/'' ***** Note: order set from CY^a''/)')
c
      else
c
c  set and compare orders from both set of functions
c
        iordp=0
        iordg=0
        do j=1,nzer
          k=kz(j)
          k1=kz1(j)
          phx1=yy(1,k)
          phy1=yy(2,k)
          phx2=yy(1,k+iasn)
          phy2=yy(2,k+iasn)
          phx11=yy1(1,k1)
          phy11=yy1(2,k1)
          phx12=yy1(1,k1+iasn)
          phy12=yy1(2,k1+iasn)
!
          phy2m=(phx1*phy2-phx2*phy1)/(phx1-phx2)
          phy12m=(phx11*phy12-phx12*phy11)/(phx11-phx12)
          dfyy=phy2m*(phx2-phx1)
          dfyy1=phy12m*(phx12-phx11)
c
c  test for same sign for both sets of eigenfunctions
c
          if(dfyy*dfyy1.gt.0) then
            if(dfyy.lt.0.and.x(k).gt.xg_lim) then
              iordp=iordp+1
              kzerp(iordp)=k
              if(icount.le.icount_p.and.istdpr.gt.0) then 
                write(istdpr,'(''p:'',i6,1p5e13.5,i6)') 
     *            k, x(k), phx1, phx2, phy2m, dfyy, iordp
                write(istdpr,'(''p:'',i6,1p5e13.5,i6)') 
     *            k1, x(k1), phx11, phx12, phy12m, dfyy1, iordp
              end if
            else if (x(k).gt.0.and.(dfyy.gt.0.or.x(k).le.xg_lim)) then
              iordg=iordg+1
              kzerg(iordg)=k
              if(icount.le.icount_p.and.istdpr.gt.0) then
                write(istdpr,'(''g:'',i6,1p5e13.5,i6)') 
     *            k, x(k), phx1, phx2, phy2m, dfyy, iordg
                write(istdpr,'(''g:'',i6,1p5e13.5,i6)') 
     *            k1, x(k1), phx11, phx12, phy12m, dfyy1, iordg
              end if
            end if
c
          else
c 
c  different phase sign. Write diagnostics and use location relative
c  to characteristic frequencies
c
            if(istdpr.gt.0) write(istdpr,'(/
     *        '' ***** For j ='',i6,'' x(k, k1) ='',1p2e13.5/
     *        ''       dfyy, dfyy1 ='',2e13.5,'' have different sign''/
     *        ''       Use position of frequency relative'',
     *        '' to sltil, bvtil'')') j,x(k),x(k1),dfyy,dfyy1
c
            i_charfreq=1
c
            if(sig.ge.sltil(k).and.sig.ge.bvtil(k).and.x(k).gt.xg_lim)
     *        then
              iordp=iordp+1
              kzerp(iordp)=k
              if(icount.le.icount_p.and.istdpr.gt.0) 
     *          write(istdpr,'(''p:'',i6,1p3e13.5,26x,i6)')
     *            k,sig,sltil(k),bvtil(k),iordp
            else
              iordg=iordg+1
              kzerp(iordp)=k
              if(icount.le.icount_p.and.istdpr.gt.0) 
     *          write(istdpr,'(''g:'',i6,1p3e13.5,26x,i6)')
     *            k,sig,sltil(k),bvtil(k),iordg
            end if
c
          end if
        end do
        if(istdpr.gt.0.and.i_charfreq.eq.1) 
     *    write(istdpr,'(//'' ***** Note:'',
     *      '' order partly based on characteristic frequencies.''//)')
c
      end if
      if (iordp.GE.iordg) then 
        iord=iordp-iordg+1
      else 
        iord=iordp-iordg
      end if
c
c  reset for very low-frequency modes, applied when istsbc = 2
c
      if(istsbc.eq.2.and.yy(1,nn)*yy(2,nn).le.0) then
c..        write(istdpr,*) '#D# x, y_1, y_2',x(nn),yy(1,nn),yy(2,nn)
        if(istdpr.gt.0) write(istdpr,
     *    '(/'' Add g node for low-frequency mode in s/r takata'')')
        iord=iord-1
        iordg=iordg+1
      end if
c
c  store p and g contributions
c
      nordp=iordp
      nordg=iordg
c
      if(icount.le.1) then
        y1s=y(1,nn)
c..        write(99,'(i2,1p9e13.5)')
c..     *    (icount,x(n),y(1,n)/y1s,y(2,n)/y1s,
c..     *    yy(1,n)/y1s,yy(2,n)/y1s,yy1(1,n)/y1s,yy1(2,n)/y1s,
c..     *    sltil(n),bvtil(n),n=1,nn)
      end if
c..      if(icount.le.4) write(99,'(i2,1p10e13.5)')
c..     *  (icount,x(n),y(1,n),y(2,n),rho(n),JJ(n),gr(n),dphipdr(n),T2(n),
c..     *  yy(1,n),yy(2,n),n=1,nn)
!
      return
      end
