      subroutine trnmod(x,aa,nn0,iaa,xtrnct,xtrnsf,ntrnct,
     *  xtrnc1,ntrns1,nw1,nibc,nn,fulmod)
c
c  sets truncation point to truncate model at x = xtrnct, taking into
c  account the possible needs of Richardson extrapolation.
c  May also set parameters to truncate the model in the atmosphere.
c  If xtrnsf .gt. 0, truncate model xtrnsf points from the surface.
c  If xtrnsf .lt. 0, truncate model at x = abs(xtrnsf), interpolating to
c  in model to this point.
c
c  With interior truncation the actual mesh point and location
c  of the interior truncation point are returned in ntrnct and xtrnc1.
c  Otherwise, with surface truncation, xtrnc1 is returned as xtrnsf
c  and the actual surface truncation point is returned in ntrns1.
c
c  Original version: 5/7/95
c
c  Modified 3/11/19, introducing option of xtrnsf .lt. 0 to truncate model
c  at precise x = abs(xtrnsf).
c  Note that if there is no interior truncation, xtrnc1 now returns
c  xtrnsf.
c
      implicit double precision(a-h,o-z)
      logical fulmod, singsf
      include 'adipls.c.d.incl'
      dimension x(1),aa(iaa,1)
      common/rhsdat/ el,ell,alb,els,el1,sig,anres,perfac,data(8)
      common/cincnt/ xmnevn,xfit, fcnorm, eps, epssol, dsigmx, fsig, 
     *  dsigre,icow,iturpr,iplneq,iriche,nnwwin, moddet, itmax, irsevn, 
     *  nftmax,irsord,iekinr,itsord,imissl,imjssl,imstsl
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
c
      ntrnct=1
      xtrnc1=0
      ntrns1=0
c
      singsf = data(7).ge.0
      if(singsf) then
	nns=nn-1
      else
	nns=nn
      end if
c
c  test for truncation at surface
c
      if(xtrnsf.eq.0) then
	ntrns1=0
c
c  test for truncating xtrnsf points below the surface
c
      else if(xtrnsf.ge.1) then
	nns=nns-xtrnsf
	ntrns1=xtrnsf
c
c test for truncation at x = abs(xtrnsf)
c
      else if(xtrnsf.lt.0) then
c
        xtrns1=-xtrnsf
        if(xtrns1.gt.x(nn0)) then
          write(istdou,'(/'' Truncation point '',f12.7,
     *      '' outside top of model, x ='',f12.7/
     *      '' No truncation.'')') xtrns1, x(nn0)
          if(istdpr.gt.0.and.istdpr.ne.istdou)
     *      write(istdpr,'(/'' Truncation point '',f12.7,
     *      '' outside top of model, x ='',f12.7/
     *      '' No truncation.'')') xtrns1, x(nn0)
          ntrns1=0
        else
          n=nn0
          do while(x(n).ge.xtrns1)
            n=n-1
          end do
          n1=n+1
          ntrns1=nn0-n1
          if(abs(x(n1)-xtrns1).gt.1.d-8) then
c
c  interpolate to xtrns1
c
            fct2=(xtrns1-x(n))/(x(n1)-x(n))
            fct1=1.d0-fct2
            x(n1)=xtrns1
            do i=1,iaa
              aa(i,n1)=fct1*aa(i,n)+fct2*aa(i,n1)
            end do
          end if
c
          if(istdpr.gt.0) write(istdpr,'(/
     *      '' Model truncated at x ='',f10.7,'' aa:''/
     *      1p5e13.5)') x(n1),(aa(i,n1),i=1,5)
c
c  set xtrnc1 for inclusion in grand summary
c
          xtrnc1=xtrnsf
        end if
      end if
c
c  locate inner truncation point
c
      if(xtrnct.gt.0) then
	do 20 n=1,nn
	ntrnct=n
	if(x(n).ge.xtrnct) go to 25
   20   continue
c
      end if
c
c  test for resetting truncation points with Richardson extrapolation
c
   25 if(iriche.eq.1.and.mod(nns-ntrnct,2).eq.1) then
c
c  when truncating at the surface, take up change there
c
        if(ntrnsf.ge.1) then
          ntrns1=ntrns1+1
c
c  otherwise change inner truncation point
c
        else
          ntrnct=ntrnct+1
        end if
      end if
c
      if(ntrnct.le.1.and.x(1).le.1.e-9) then
        nw1=1
        nibc=2
        fulmod=.true.
      else
        fulmod=.false.
c
c  truncation point
c
        xtrnc1=x(ntrnct)
c
c  reset first output point and point of inner boundary condition
c
        nw1=ntrnct
        nibc=ntrnct
      end if
c
c  truncation at surface?
c
      if(ntrns1.gt.0) nn=nn-ntrns1
c
      return
      end
