#!/bin/csh
#  script to call fgong-amdl.d
if($#argv < 2) then 
	set help
else
	switch($1)
	  case "-help":
		set help
		breaksw
	  default
		breaksw
	endsw
endif

if($?help) then
        echo "Usage: fgong-amdl.d [-noform] [-xtrnc xtrnc] [-idt] <Input file> <output file> [G] [fgar]"
	echo "If -noform is set, use free format for read"
        echo "G gives the gravitational constant, in units of 1.e-8 cgs"
        echo "(default 6.67232, for ASTEC, but may be changed by setting"
        echo "the value in file CGRAV in directory where the code is run)"
	echo "This is always overridden by the value of G given in"
	echo "glob(15) in the fgong file, if non-zero"
        echo "If -xtrnc is set, truncate model at xtrnc R"
	echo "If -idt is set, eliminate points with identical r"
	echo "If fgar is set, reset Garstec spikes from density derivative"
        exit(1)
endif

switch($1)
	case "-noform":
	  set iform=0
	  shift
	  breaksw
	default:
	  set iform=1
	  breaksw
endsw

switch($1)
	case "-xtrnc":
	  set xtrnc = $2
          echo setting xtrnc to $xtrnc
	  shift
	  shift
	  breaksw
	default:
	  set xtrnc = "-1"
	  breaksw
endsw

switch($1)
	case "-idt":
	  set idt
	  shift
	  breaksw
	default:
	  set idt = "-1"
	  breaksw
endsw

set cgrav = 6.67232
if(-f CGRAV) then
  set cgrav = `get-input CGRAV`
endif

if($#argv > 2) then
        set cgrav = $3
endif

if($#argv > 3) then
	@ iform+=10
endif

if($?idt) then
	@ iform+=100
endif

(echo $1; echo $2; echo $cgrav; echo $xtrnc; echo $iform) | \
  $aprgdir/adiajobs/fgong-amdl.d.x
