#!/bin/csh
#  script to call  $prgdir/adiajobs/set-fobs.d.x
#  usage (with arguments) set-obs.d <case> <input file> [<output file>]
#  If output file is not given, assumes that <input file> contains 
#  trailer to be applied to agsm on input and obs on output.
#  with no arguments: prompts

if($#argv == 1) then
  echo "usage (with arguments) "
  echo "set-fobs.d [-cgrav cgrav] <case> <input file> [<output file>]"
  echo "If output file is not given, assumes that <input file> contains "
  echo "trailer to be applied to agsm on input and obs on output."
  echo "with no arguments: prompts"
  echo "case:"
  echo "1: grand summary, variational frequency."
  echo "4: grand summary, from eigenfrequency in cs(20)."
  echo "   Note that this allows setting Cowling approximation frequency"
  echo "5: grand summary, from Richardson extrapolation frequency"
  echo "6: grand summary, from (possibly corrected) eigenfrequency in cs(21)"
  echo "If -cgrav is set, cgrav gives gravitational constant, in units of 1.d-8"
  exit(1)
endif

set cgrav = 6.67232
if(-f CGRAV) then
  set cgrav = `get-input CGRAV`
  echo cgrav set to $cgrav
endif

switch($1)
	case -cgrav:
	  set cgrav = $2
	  shift
	  shift
	  breaksw
	default:
	  breaksw
endsw

if($#argv < 2) then
	$aprgdir/adiajobs/set-fobs.d.x
else if($#argv == 2) then
	(echo $1; echo agsm.$2; echo fobs.$2; echo $cgrav) | \
	$aprgdir/adiajobs/set-fobs.d.x
else
	(echo $1; echo $2; echo $3; echo $cgrav) | \
	$aprgdir/adiajobs/set-fobs.d.x
endif
