      subroutine   lirt(z,zi,y,yi,ii,id,nt,l,inter,icase)
c
c
c
c                interpolation/extrapolation routine
c
c
c     for a such that z=zi(a),  sets y(i)=yi(i,a), i=1,ii
c
c     zi(n),yi(i,n) must be supplied for n=1,nt and i=1,ii
c     id is first dimension of yi
c
c     inter is set to 1 for interpolation and 0 for extrapolation
c     inter is returned as -1 in case of errors
c
c     if l.le.1, scan to find the zi(n) which immediately bound z
c                starts at n=1
c     if l.gt.1, scan starts from value of n from previous call of lir
c
c
c     lir use cubic interpolation/extrapolation unless nt.lt.4
c
c     modified 6/4/13 to test for uneven mesh spacing and use of 
c     linear interpolation in such regions. This is flagged by returning
c     inter = 10
c
c     Modified 1/4/14 introducing flag icase to control test for 
c     linear interpolation.
c     icase = 1: test on uneven mesh spacing as before.
c     icase = 2: in addition, test for corner in dependent variable.
c
c  Double precision version.
c  ++++++++++++++++++++++++
c
c  Dated 10/3/90.
c
c  Note: this double precision version of the routine has same name
c  as single precision version, but is distinguished by its file name
c
c
      implicit double precision(a-h,o-z)
      dimension zi(*),y(*),yi(*),a(4),dd(3)
c
c  limit on mesh-spacing ratios for switching to linear interpolation
c
      common/c_lirt/ dxy_lim, dsc_lim
c
c  common defining standard input and output
c
      common/cstdio/ istdin, istdou, istdpr, istder
      save
      data n/-1/
      data idiag /3/
c
c  set scale for test on corner in dependent variable
c
      zscale=0.5d0*(abs(zi(1))+abs(zi(nt)))
c
      il=0
    1 continue
      ir=1
c
c     check nt and reset il if necessary
      if(nt.lt.2) go to 101
      if(nt.lt.4) il=1
c
c     addressing constants
      inter=1
      ir1=ir-1
      ird=ir*id
      iir=(ii-1)*ir+1
      j=(nt-1)*ir+1
      diff=zi(j)-zi(1)
c
c     set index for start of search
      n=(n-2)*ir+1
      if(l.le.1.or.n.lt.1) n=1
c
c     determine position of z within zi
    2 if(n.gt.j) go to 8
      if(diff) 4,102,3
    3 if(zi(n)-z) 5,6,9
    4 if(zi(n)-z) 9,6,5
    5 n=n+ir
      go to 2
c
c     set y when z lies on a mesh point
    6 j=(n-1)*id
      do 7 i=1,iir
      y(i)=yi(i+j)
c..      write(istdpr,*) '#D# i, y', i,y(i)
    7 if(y(i).eq.0.d0) y(i+ir1)=0.d0
      go to 30
c
c     control when z does not lie on a mesh point
    8 inter=0
    9 if(n.le.1) inter=0
      if(il.eq.1) go to 20
c
c     cubic interpolation/extrapolation
c
c     pivotal point (m) and point (k) closest to z
   10 m=n
      k=3
      if(n.gt.1+ir) go to 11
      m=1+ir+ir
      k=n
   11 if(n.lt.j) go to 12
      m=j-ir
      k=4
c
c
c     weighting factors
   12 y1=zi(m-ir*2)
      y2=zi(m-ir)
      y3=zi(m)
      y4=zi(m+ir)
c
      if(icase.ge.1) then
c
c  test for excessive jump in mesh spacing
c
        dy12=abs((y2-y1)/(y3-y2))
        dy23=abs((y3-y2)/(y4-y3))
        if(dy12.lt.1d0/dxy_lim.or.dy12.gt.dxy_lim.or.
     *     dy23.lt.1d0/dxy_lim.or.dy23.gt.dxy_lim) then 
          if(idiag.gt.1) then
            if(istdpr.gt.0.and.istdpr.ne.istdou) write(istdpr,
     *        '('' **** In lirt excessive jump in mesh spacing at x ='',
     *         1pe16.8/''      Use linear interpolation'')') z
            write(istdou,
     *      '('' **** In lirt excessive jump in mesh spacing at x ='',
     *      1pe16.8/''      Use linear interpolation'')') z
c..            write(istdpr,*) y1,y2,y3,y4
          end if
          inter=10
          go to 20
        end if
      end if
c
      if(icase.ge.2.and.m.ge.3.and.m.le.nt-1) then
c
c  test for corner in dependent variable
c
        m1=(m-1)/ir-3
        m1=m1*ird
        do i1=1,iir
          k1=i1+m1
          d21=abs((yi(k1+2*ird)-yi(k1+ird))/(y2-y1))
          d32=abs((yi(k1+3*ird)-yi(k1+2*ird))/(y3-y2))
          d43=abs((yi(k1+4*ird)-yi(k1+3*ird))/(y4-y3))
          dd(1)=d21
          dd(2)=d32
          dd(3)=d43
          dmax=max(d21,d32,d43)
          dmin=min(d21,d32,d43)
          do j1=1,3
            if(dd(j1).eq.dmin) jmin=j1
            if(dd(j1).eq.dmax) jmax=j1
          end do
          do j1=1,3
            if(j1.ne.jmin.and.j1.ne.jmax) dint=dd(j1)
          end do
          dscale=max(abs(yi(k1+ird)),abs(yi(k1+2*ird)),
     *               abs(yi(k1+3*ird)),abs(yi(k1+4*ird)))/zscale
          if(y1.ge.258.and.y1.le.310.and.i1.eq.-4) then
            write(istdpr,*) '#D# z, y1 - y4:'
            write(istdpr,*) z, y1,y2,y3,y4
            write(istdpr,*) '#D# yi(1-4):'
            write(istdpr,*) (yi(k1+j1*ird),j1=1,4)
            write(istdpr,*) '#D# y1, d21, d32, d43, zscale, dscale'
            write(istdpr,*) y1, d21, d32, d43, zscale, dscale
            write(istdpr,*) '#D# jmin, jmax, dint =',jmin, jmax, dint
          end if
          if((d21.ge.dxy_lim*d32.or.d32.ge.dxy_lim*d21.or.
     *      d32.ge.dxy_lim*d43.or.d43.ge.dxy_lim*d32).and.
     *      dmax.gt.dsc_lim*min(dint,dscale)) then
            if(idiag.gt.1) then
              if(istdpr.gt.0.and.istdpr.ne.istdou)
     *          write(istdpr,
     *            '('' **** Corner in dependent variable'',i3,
     *            '' at x ='',1pe16.8/
     *            ''      Use linear interpolation'')') i1, z
              write(istdou,
     *          '('' **** Corner in dependent variable'',i3,
     *          '' at x ='',1pe16.8/
     *          ''      Use linear interpolation'')') i1, z
              if(istdpr.gt.0.and.idiag.gt.2) 
     *          write(istdpr,'(/ '' n, x, y, dn+1n:''/
     *          (i6,1p3e15.7))')
     *          m-2,zi(m-2),yi(k1+  ird),d21,
     *          m-1,zi(m-1),yi(k1+2*ird),d32,
     *          m  ,zi(m  ),yi(k1+3*ird),d43,
     *          m+1,zi(m+1),yi(k1+4*ird),dscale
            end if
            inter=15
            go to 20
          end if
        end do
      end if
c
c  Ordinary cubic interpolation
c
      z1=z-y1
      z2=z-y2
      z3=z-y3
      z4=z-y4
c..      write(istdpr,*) '#D# z, zi:',z1,z2,z3,z4
c
   13 z12=z1*z2
      z34=z3*z4
c
   14 a(1)=z2*z34/((y1-y2)*(y1-y3)*(y1-y4))
      a(2)=z1*z34/((y2-y1)*(y2-y3)*(y2-y4))
      a(3)=z12*z4/((y3-y1)*(y3-y2)*(y3-y4))
      a(4)=z12*z3/((y4-y1)*(y4-y2)*(y4-y3))
c..      write(istdpr,*) '#D# a(1-4):',a
c
c     correct a(k)
   15 diff=a(1)+a(2)+a(3)+a(4)
c..      write(istdpr,*) '#D# k, diff,diff-1.d0', k, diff,diff-1.d0
      a(k)=(1.d0+a(k))-diff
c..      write(istdpr,*) '#D# a(1-4):',a
c
c     compute y
   16 m=(m-1)/ir-3
      m=m*ird
      do 18 i=1,iir
      k=i+m
      yy=0.d0
      do 17 j=1,4
      k=k+ird
      diff=yi(k)
   17 yy=yy+a(j)*diff
      y(i)=yy
   18 if(y(i).eq.0.d0) y(i+ir1)=0.d0
      go to 30
c
c     linear interpolation/extrapolation
   20 if(n.eq.1) n=1+ir
      if(n.gt.j) n=j
      z1=zi(n)
      y1=(z1-z)/(z1-zi(n-ir))
      y2=1.d0-y1
      j=(n-1)*id
      m=j-ird
      do 21 i=1,iir,ir
      y(i)=y1*yi(i+m)+y2*yi(i+j)
   21 if(y(i).eq.0.d0) y(i+ir1)=0.d0
c
c     reset n
   30 n=(n+ir-1)/ir
      return
c
c
c     diagnostics
  101 if(istdpr.gt.0) write(istdpr,1001) nt
      inter=-1
      return
  102 if(istdpr.gt.0) write(istdpr,1002) zi(1),nt,zi(j)
      inter=-1
      return
c
 1001 format(/1x,10('*'),5x,'there are fewer than two data points in',
     *      ' lir     nt =',i4,5x,10('*')/)
 1002 format(/1x,10('*'),5x,'extreme values of independent variable',
     *      ' equal in lir',5x,10('*')/16x,'zi(1) =',1pe13.5,',   ',
     *       'zi(',i4,') =',1pe13.5/)
c
      end
      block data bllirt
      double precision dxy_lim, dsc_lim
c
      common/c_lirt/ dxy_lim, dsc_lim
c
c  limit on mesh-spacing ratios for switching to linear interpolation
c
      data dxy_lim, dsc_lim /3.d0,100.d0/
      end
