pro psend, file = psfile, printer = psprinter, remote = remote, $
  putid = putid, gzip = gzip, nocompress = nocompress
;
;  If file is set, the ps file is put in psfile, which is compressed
;  and kept after the call.
;  Otherwise the output is sent to idl.ps, which is deleted.

;  psprinter may be used to designate alternative printer
;  to the default
;  If psprinter = 'null', no print is produced, and the ps file
;  is kept.

;  If remote is set, the print is routed, by remote shell, to 
;  the system remote. In that case, psprinter must contain the
;  complete print command on the remote system, appropriate for
;  printing from standard input

; If putid is set, put identifying stamp (user@host and date) on plot

; If gzip is set, ps file is gzipped, otherwise it is compressed,
; unless nocompress is set.

;  Set default print command and parameters. Remember to include blanks

;  Appropriate to Aarhus HP system:
; lpcom = 'lp'
; defpar = ' -c -d'
; defprt = ' -c -dps'

;  Appropriate to HAO Sun systems:
lpcom = 'lpr'
defpar = ' -h -P'
defprt = 'hpast02'

; Remote shell command

rsh = 'rsh'


IF keyword_set(putid) THEN BEGIN
  spawn, ['uname', '-n'], host, /noshell
  host = host(0)
  user = getenv('USER')
  id = user+'@'+host+' '+systime()
  xyouts, 1, 0.013, /normal, id, charsize = 0.5, /alignment
ENDIF

;
;	device,/close

	if keyword_set(gzip) then compress='gzip -f ' else $
	if keyword_set(nocompress) then compress='echo no compress of ' else $
				compress='compress -f '
	;print,'compress =',compress
	if keyword_set(remote) then begin
	  if keyword_set(psprinter) then begin
	    if psprinter eq 'null' then begin 
	      if keyword_set(psfile) then $
		lp = compress + psfile + '; echo "No print"' $
		  else lp = compress + ' idl.ps ; echo "No print"'
	    endif else begin
	      if keyword_set(psfile) then $
	        lp = 'cat ' + psfile + ' | compress | ' + $
                  rsh +' '+ remote +' '' uncompress | '+ psprinter + '''' + $
		  '; ' + compress + psfile $
	      else $
	        lp = 'cat idl.ps | compress |' + $
		  rsh +' '+ remote +' ''uncompress |'+ psprinter + '''' +$
		  '; rm idl.ps'
	    endelse
            print,lp
	    spawn,lp
            return
	  endif else begin
	    print,' *** Error in psend. remote set without psprinter.'
	    print,'     Print to standard local printer'
	  endelse
	endif
	    
        lp = lpcom
	if keyword_set(psprinter) then begin
	  if psprinter eq 'null' then $
	    lp = 'null' else if psprinter eq 'phaser' then $
	    lp = 'xprint -n -dphaser2' else $
	    lp = lp + defpar + psprinter
	endif else lp = lp + defpar + defprt

	if lp eq 'null' then begin
	  if keyword_set(psfile) then $
	    lp = compress + psfile + '; echo "No print"' $
	      else lp = compress + ' idl.ps ; echo "No print"'
	endif else begin
	  if keyword_set(psfile) then $
	    lp = lp + ' ' + psfile + '; ' + compress + psfile $
	  else lp = lp + ' idl.ps; rm idl.ps'
	endelse
	print, lp
	spawn, lp
return
end
